/*****************************************************************
	T/SDAS-4215 demo/test utility.
	(C) MicroLAB Systems Ltd, 1994-1998

******************************************************************/


/* PC-to-DSP communication descriptor */

struct	MCD_DD
	{ int	m_id,
		prg_id,
                status,
                err,
		cmnd;
	  int	fq;
	  int   epoch_len,
                adc_data_ptr,
                dac_data_ptr,
		chan ,

			/* S4215 specific parameters for DMODE_B4..B7 of real-time DATA mode */
		lo,	/* left output attenuation   (0..63) */
		ro,	/* right output attenuation  (0..63) */
		lg,	/* left input gain           (0..15) */
		rg,	/* right input gain          (0..15) */
		ma,	/* monitor path attenuation  (0..15) */
		le,	/* line output enable        (0/1) */
		he,	/* headphone output enable   (0/1) */
		is;	/* input selector (0-line input, 1-mic input) */
	} ;
/*	default address for MCD in SRAM is *(0x40) */	
struct	MCD_DD *MCD	= (struct MCD_DD *)0x40;


#define	T31_ID			0x101
#define	PRG_ID			0x100

#define	MSTAT_RDY		0 /* Master status values */
#define	MSTAT_BUSY		1
#define	MSTAT_ATTN		2
#define	MSTAT_GO		3
#define	MSTAT_ABORT		4

#define	MERR_OK			0 /* Master returned error codes */
#define	MERR_INV_CMND		1 /*     ; invalid command */
#define	MERR_INV_ATTN		2
#define	MERR_HW_ERR		3

#define	MCMND_NOP		0
#define	MCMND_ABORT		1

#define	MCMND_DAS_SCAN		0x100
#define	MCMND_DAS_DAC_SCAN	0x101
#define	MCMND_DAC_SING_LOAD	0x102
#define	MCMND_ADC_TO_DAC	0x103

#define	ML_OFF			0
#define	ML_ON			1

#define	MDATA_BUF_LEN		(0x7C00L/4) /* maximum data array length */


extern  int adc_data_start;
extern	int dac_data_start;
int	*ADC,	*DAC;		/* ptrs to the ADC/DAC data */

int	host_rq_flag=0;


#include	<math.h>

#include	"c30_def.h"
#include	"t3_4215.i"

void c_int30()
	{ host_rq_flag=1;
   }

cpu_init()
 {
	   *STB0_C = 0xf0600;	/* set PRIMARY/SECONDARY bus control registers */
	   *STB1_C = 0xf0600;
	   asm("  ldi 6,iof");	/* set MLOCK bit */

        INT_EDGE;
	CACHE_ENABLE;
 }





int	cmnd_decoder()			/* command decoder	*/
 {
	if(MCD->status!=MSTAT_ATTN) return(MERR_INV_ATTN);

	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
	   case MCMND_NOP:		/* common cmnds */
	   case MCMND_ABORT:
		 return(MERR_OK);

                                        /* application specific commands */
	   case MCMND_DAS_DAC_SCAN:
	   	 return( das_dac_scan() );

	   case MCMND_ADC_TO_DAC:
	   	 return( adc_to_dac() );

	   default:			/* all unknown commands cause error */
		 return( MERR_INV_CMND );
	 }

 }






void main()
{
	cpu_init();

	ADC	=   (int *) &adc_data_start ;
	DAC	=   (int *) &dac_data_start ;

	MCD->m_id		= T31_ID;
	MCD->prg_id		= PRG_ID;
	MCD->cmnd		= 0;
	MCD->fq			= 0;
	MCD->epoch_len		= 0;
	MCD->err		= MERR_OK;
	MCD->adc_data_ptr	= (int) ADC;
	MCD->dac_data_ptr	= (int) DAC;
	MCD->status		= MSTAT_RDY;

	while(1)			/* continuous loop	*/
	 {
	   host_rq_flag=0;		/* poll for the HOST RQ */

	   EXT_IRQ_CLEAR(3);
	   EXT_INT_ENABLE(3);

	   SX0_INT_DISABLE;
	   SR0_INT_DISABLE;

	   INT_ENABLE;

           while(host_rq_flag==0) ;

	   EXT_INT_DISABLE(3);		/* disable HOST RQ	*/

	   MCD->err = cmnd_decoder();	/* call exec & return error code */
           MCD->status = MSTAT_RDY;	/* clear status flag */

	 }
}
