/***************************************************************
	Application specific execution module.
	device: T/SDAS-4215 for TORNADO-54x
****************************************************************/

/*
   Control/Data time slots for CS4215 Audio-codec chip.
   Organized as 16-bit words for portability between 32-bit and 16-bit DSPs.
   The lower significant time slots should be allocated into MSB of 16-bit word.
*/

/*
  Declaration of TLC command data structures.
  Note:   the TI C54x C Compiler processes bit fields in the reverse manner, i.e.
	  first declared fields appear in the most significant bits of the structure
	  word. (see TI C54x C Compiler UG, section 5.1.7 (Field/Structure Alignment).
	  Note that TI C3x C Compiler provides bit fields processing in the
	  diret manner (!!!).
*/


typedef struct     	/* control time slots 1 & 2 */
	{
	  unsigned int rsrv3  :3; /* should be ==001 always */
	  unsigned int mlb    :1;
	  unsigned int olb    :1;
	  unsigned int clb    :1;
	  unsigned int rsrv2  :2; /* CTS1: reserved bits */
	  unsigned int hpf    :1;
	  unsigned int rsrv1  :1;
	  unsigned int dfr    :3; /*        DFR-0..2*/
	  unsigned int st     :1;
          unsigned int df     :2; /* CTS2:  DF-0/1 */
	} _AC_CTS_1_2;

typedef struct     	/* control time slots 3 & 4 */
	{
	  unsigned int its    :1;
	  unsigned int mck    :3;
	  unsigned int bsel   :2;
	  unsigned int xclk   :1;
	  unsigned int xen    :1; /* CTS3 */
	  unsigned int test   :6;
	  unsigned int enl    :1;
          unsigned int dad    :1; /* CTS4*/
	} _AC_CTS_3_4;

typedef struct     	/* control time slots 5 & 6 */
	{
	  unsigned int pio    :2;
	  unsigned int rsrv2  :6; /* CTS5*/
          unsigned int rsrv1  :8; /* CTS6 is reserved*/
	} _AC_CTS_5_6;

typedef struct     	/* control time slots 7 & 8 */
	{
	  unsigned int rsrv2  :4;
	  unsigned int ver    :4; /* CTS7*/
	  unsigned int rsrv1  :8; /* CTS8 is reserved*/
	} _AC_CTS_7_8;


typedef struct     	/* data time slots 1 & 2 */
	{ int data_l  :16; /* DTS-1/2, left ADC/DAC data */
	} _AC_DTS_1_2;

typedef struct     	/* data time slots 3 & 4 */
	{ int data_r  :16; /* DTS-3/4, right ADC/DAC data */
	} _AC_DTS_3_4;

typedef struct     	/* data time slots 5 & 6 */
	{
	  unsigned int he    :1;
	  unsigned int le    :1;
	  unsigned int lo    :6; /* DTS-5 */
	  unsigned int adi   :1;
	  unsigned int se    :1;
          unsigned int ro    :6; /* DTS-6, right output attenuation */
	} _AC_DTS_5_6;

typedef struct     	/* data time slots 7 & 8 */
	{
	  unsigned int pio   :2;
	  unsigned int ovr   :1;
	  unsigned int is    :1;
	  unsigned int lg    :4; /* DTS-7 */
	  unsigned int ma    :4;
          unsigned int rg    :4; /* DTS-8 */
	} _AC_DTS_7_8;



_AC_CTS_1_2	t_ac_cts_1_2;
_AC_CTS_3_4	t_ac_cts_3_4;
_AC_CTS_5_6	t_ac_cts_5_6;
_AC_CTS_7_8	t_ac_cts_7_8;

unsigned int    cmode_w0,	/* transmitted 16-bit control time slots */
                cmode_w1,
                cmode_w2,
                cmode_w3,
		cmode_rd_w0,	/* received 16-bit control time slots */
		cmode_rd_w1,
		cmode_rd_w2,
		cmode_rd_w3 ;


_AC_DTS_5_6	t_ac_dts_5_6;
_AC_DTS_7_8	t_ac_dts_7_8;

unsigned int    dmode_w2,	/* transmitted 16-bit data time slots */
                dmode_w3,
                dmode_rd_w2,
		dmode_rd_w3;	/* received 16-bit data time slots */



#define	_cmode_rd_w0_mask	0x0c3f /* AND mask for CMODE_W0 to mask reserved bits */
#define	_cmode_rd_w1_mask	0xbf03 /* AND mask for CMODE_W1 to mask reserved bits */
#define _cmode_rd_w2_mask	0x0000 /* AND mask for CMODE_W2 to mask reserved bits */ 
#define _cmode_rd_w3_mask	0x0000 /* AND mask for CMODE_W3 to mask reserved bits */ 


#define	_cmode_CLB_mask	0x0400 /* CLB bit mask applies to CMODE_W0*/


/*-------- MACROS for setting CS4215 CONTROL/DATA mode via TIMER-1 line */
#define	SET_AC_CMODE		{T54X_set_xiof1(0);rx_mode=RX_CMODE;}
#define	SET_AC_DMODE		{T54X_set_xiof1(1);rx_mode=RX_DMODE;}

#define CLEAR_CLB		(cmode_w0 &=~_cmode_CLB_mask);
#define SET_CLB			(cmode_w0 |= _cmode_CLB_mask);


/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

        The tracking signal is XIOF-0/TM signal from the SIOX site connector.
	This shared signal should be configured by on-board jumper J3
	as the XIOF-0 signal.

	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to pin XIOF-0 of SIOX connector.
	(refer to TORNADO-54x documentation re SIOX signal specs).

*/

#define _TIME_TRACKING_SET_0	T54X_set_xiof0(0)
#define _TIME_TRACKING_SET_1	T54X_set_xiof0(1)



		/* X/R interrupt handler parameters	*/

unsigned int dac_scan_flag		= ML_OFF,
	dac_cyclic_scan_flag	= ML_ON,
	dac_data_start_ptr,
        dac_get_ptr,
	dac_data_count,
        dac_data_count_max,
	x_cmode			= 0;


unsigned int adc_scan_flag		= ML_OFF,
	adc_cyclic_scan_flag	= ML_OFF,
	adc_save_ptr,
	adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max,
	r_cmode			= 0;


int	rx_mode	= 0;    	/* C-MODE or D-MODE */
int	scan_channels=0;	/* channels for SCAN */

#define	RX_CMODE	0
#define RX_DMODE	1

#define CHAN_0		1	/* scanned channel IDs: left-channel */
#define CHAN_1		2	/* right channel */


#define WAIT_CMODE_RX_OK	while(x_cmode!=0 || r_cmode!=0);



/*----------- transmitter interrupt --------------*/

int 		_x_wcount=0;	/* transmitter word counter */
unsigned	dxr;		// data to be transmitted

interrupt void c_bsp_xint()   /* BSP transmitter interrupt */
{
//   _TIME_TRACKING_SET_1;	/* set PERFORMACE TRACKING FLAG */

   if(rx_mode==RX_CMODE)	/* CONTROL MODE */
      {
        if(x_cmode==ML_ON)
	 {
	  switch(_x_wcount)     /* decode word # and transmit the word */
	   { case 0:
		  dxr=cmode_w0;
		  break;

             case 1:
		  dxr=cmode_w1;
		  break;

             case 2:
		  dxr=cmode_w2;
		  break;

             case 3:
		  dxr=cmode_w3;
		  x_cmode=ML_OFF;		/* clear X_CMODE flag for background control */
		  break;
	   }
          if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
		   *C54X_MCBSP0_DXR1=dxr;
	     else  *C54X_BSP_DXR=dxr;

	 }
       }
      else			/* DATA mode */
       {
	  switch(_x_wcount)     /* decode word # and transmit the word */
	   { case 0:	/* transmit DAC channel #0 */
		  dxr = ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0) ? *((int*)dac_get_ptr):0;
		  break;

             case 1:	/* transmit DAC channel #1 */
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0) ? *((int*)(dac_get_ptr+dac_data_count_max)):0;

			/* update pointers and counters */
		if(dac_scan_flag==ML_ON)
                { dac_get_ptr++;
                  if(++dac_data_count >=dac_data_count_max)
       	            { if(dac_cyclic_scan_flag==ML_ON)
		      {
			dac_data_count=0;
                        dac_get_ptr=dac_data_start_ptr;
		      }
		      else
		        dac_scan_flag=ML_OFF;
		     }
		}
		  break;


             case 2:	/* transmit DMODE_W2 */
		  dxr=dmode_w2;
		  break;

             case 3:	/* transmit DMODE_W3 */
		  dxr=dmode_w3;
		  break;
	   }

          if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
		    *C54X_MCBSP0_DXR1=dxr;
	      else  *C54X_BSP_DXR=dxr;

       }

        _x_wcount++;		/* increment transmitter word counter */
	_x_wcount %=4;

//   _TIME_TRACKING_SET_0;        /* clear PERFORMACE TRACKING FLAG */

}


/*----------- receiver interrupt --------------*/

int _r_wcount=0;	/* receiver word counter */
int _r_data;

interrupt void c_bsp_rint()   /* BSP receiver interrupt */
{
//   _TIME_TRACKING_SET_1;	/* set PERFORMACE TRACKING FLAG */

   _r_data= (SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR) ;		/* read receiver data */


   if(rx_mode==RX_CMODE)	/* CONTROL MODE */
      {
        if(r_cmode==ML_ON)
	 {
	  switch(_r_wcount)     /* decode word # and save the word */
	   { case 0:
		cmode_rd_w0=_r_data;	   	
		break;

	     case 1:
		cmode_rd_w1=_r_data;	   	
		break;

	     case 2:
		cmode_rd_w2=_r_data;	   	
		break;

	     case 3:
		cmode_rd_w3=_r_data;	   	
                r_cmode=ML_OFF;		/* clear R_CMODE flag for background control */
		break;
	    }
	  }
       }
      else			/* DATA mode */
       {
	  switch(_r_wcount)     /* decode word # and save the word */
	   { case 0:	/* save as ADC channel #0 */
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   *((int *)adc_save_ptr) = _r_data;
	      break;

	     case 1:	/* save as ADC channel #1 */
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   *((int *)(adc_save_ptr+adc_data_count_max)) = _r_data;

			/* update pointers and counters */
	      if(adc_scan_flag==ML_ON)
                { adc_save_ptr++;
                  if(++adc_data_count >=adc_data_count_max)
		    { if(adc_cyclic_scan_flag==ML_ON)
		      {
			adc_data_count=0;
                        adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		    }
		}
	       break;


	     case 2:	/* save as DMODE_RD_W2 */
	       dmode_rd_w2 = _r_data;
	       break;

	     case 3:	/* save as DMODE_RD_W3 */
	       dmode_rd_w3 = _r_data;
	       break;
	   }
        }

        _r_wcount++;		/* increment receiver word counter */
	_r_wcount %=4;

//   _TIME_TRACKING_SET_0;        /* clear PERFORMACE TRACKING FLAG */
}



/*>>>>>>>>>> config data for SIO port <<<<<<<<<<<<<*/

#define	BSP_SPC_GO_MASK		0x00c0


/*----- SIO config for CONTROL MODE -----------
  BSP:
	FSX=out XCLK_src=int RCLK_src=ext
	XFSM=RFSM=contin  CLKRP=noram
	XLEN=RLEN=16
	DIV=31-> CLKX=CLKOUT/32

  McBSP:
	FSX=out XCLK_src=int RCLK_src=ext
	XLEN=RLEN=16  XINT=RINT=enable
        FRAME SYNC generator = 100

*/

#define	BSP_SPC_CMODE			0x0030
#define	BSP_SPCE_CMODE			0x011F

#define	MCBSP_SPCR1_CMODE_GO_MASK	0x0001
#define	MCBSP_SPCR2_CMODE_GO_MASK	0x0041
#define	MCBSP_SPCR1_GO_FRAME_MASK	0x0000
#define	MCBSP_SPCR2_GO_FRAME_MASK	0x0080

#define	MCBSP_SPCR1_CMODE		0x0000
#define	MCBSP_SPCR2_CMODE		0x0000

#define	MCBSP_PCR_CMODE			0x0a00

#define	MCBSP_XCR1_CMODE		0x0340
#define	MCBSP_XCR2_CMODE		0x0005
#define	MCBSP_RCR1_CMODE		0x0340
#define	MCBSP_RCR2_CMODE		0x0005

#define	MCBSP_SRGR1_CMODE		0x0028	// CLKper=400ns, FSPER=512
#define	MCBSP_SRGR2_CMODE		0x3200


void init_sio_cmode()
 {
   if(SIO_MCBSP)	//-- configure C54xx DSPs with McBSP ports
    {
 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = MCBSP_PCR_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = MCBSP_RCR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = MCBSP_RCR2_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = MCBSP_XCR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = MCBSP_XCR2_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP0_SRGR1 = MCBSP_SRGR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP0_SRGR2 = MCBSP_SRGR2_CMODE;

		//-- now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_CMODE_GO_MASK;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_CMODE_GO_MASK;


	   *C54X_MCBSP0_DXR1  = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
     else	//-- configure C548/C549/C542 DSP with BSP/TDM ports
    {
	   *C54X_BSP_SPC  = BSP_SPC_CMODE;			/* reset BSP */
	   *C54X_BSP_SPCE = BSP_SPCE_CMODE;			
	   *C54X_BSP_SPC  = BSP_SPC_CMODE | BSP_SPC_GO_MASK;	/* enable BSP */
    }

 }


	// macros to enable/disable FSG during CMODE transmission
#define MCBSP_FSX_ENABLE	{*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;*C54X_MCBSP0_SPCR2 |= MCBSP_SPCR2_GO_FRAME_MASK;}
#define	MCBSP_FSX_DISABLE	{*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;*C54X_MCBSP0_SPCR2 &= ~MCBSP_SPCR2_GO_FRAME_MASK;}


/*----- SIO config for DATA MODE ---------

  BSP:
  	FSX=out XCLK_src=RCLK_src=ext
	XFSM=RFSM=contin
	XLEN=RLEN=16

  McBSP:
        RJUST=00
	FSX/FSR=in XCLK_src=RCLK_src=ext
	XLEN=RLEN=16  XINT=RINT=enable
*/

#define	BSP_SPC_DMODE			0x0000
#define	BSP_SPCE_DMODE			0x0100


#define	MCBSP_SPCR1_DMODE_GO_MASK	0x0001
#define	MCBSP_SPCR2_DMODE_GO_MASK	0x0001

#define	MCBSP_SPCR1_DMODE		0x0000
#define	MCBSP_SPCR2_DMODE		0x0000
#define	MCBSP_PCR_DMODE			0x0000
#define	MCBSP_XCR1_DMODE		0x0340
#define	MCBSP_XCR2_DMODE		0x0005
#define	MCBSP_RCR1_DMODE		0x0340
#define	MCBSP_RCR2_DMODE		0x0005

#define	MCBSP_SRGR1_DMODE		0x0000
#define	MCBSP_SRGR2_DMODE		0x0000


void init_sio_dmode()
 {

   if(SIO_MCBSP)	//-- configure C54xx DSPs with McBSP ports
    {
 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = MCBSP_PCR_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = MCBSP_RCR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = MCBSP_RCR2_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = MCBSP_XCR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = MCBSP_XCR2_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP0_SRGR1 = MCBSP_SRGR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP0_SRGR2 = MCBSP_SRGR2_DMODE;

		//-- now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DMODE_GO_MASK;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DMODE_GO_MASK;

    }
     else	//-- configure C548/C549/C542 DSP with BSP/TDM ports
    {

	   *C54X_BSP_SPC  = BSP_SPC_DMODE;			/* reset BSP */
	   *C54X_BSP_SPCE = BSP_SPCE_DMODE;			
	   *C54X_BSP_SPC  = BSP_SPC_DMODE | BSP_SPC_GO_MASK;	/* enable BSP */
    }

 }


void set_ac_config_data()
{
	/* create CMODE words #0/#1 and DMODE word #1 */

	t_ac_cts_1_2.clb=0;     /* CLB bit =0 */
	t_ac_cts_1_2.olb=0;	/* select 1Vrms scale */
	t_ac_cts_1_2.mlb=0;	/* enable MIC +20dB amplifier */

        t_ac_cts_1_2.rsrv3=1;	/* reserved bits */
	t_ac_cts_1_2.rsrv2=0;
	t_ac_cts_1_2.rsrv1=0;

	t_ac_cts_1_2.df = 0;	/* 16-bit data */
	t_ac_cts_1_2.st	= 1;	/* stereo mode */
	t_ac_cts_1_2.hpf = 0;	/* disable high-pass filter */

				/* set CLK selection */
	t_ac_cts_1_2.dfr = MCD->fq & 0x7;
	t_ac_cts_3_4.mck = (((MCD->fq)>>3) & 0x1) +1;

	t_ac_cts_3_4.xen = 0;
	t_ac_cts_3_4.xclk= 1;
	t_ac_cts_3_4.bsel= 0;
	t_ac_cts_3_4.its = 1;
	t_ac_cts_3_4.dad = 0;	/* disable loopback and tests */
	t_ac_cts_3_4.enl = 0;
	t_ac_cts_3_4.test= 0;

	t_ac_cts_5_6.rsrv1 =0;
	t_ac_cts_5_6.rsrv2 =0;
	t_ac_cts_5_6.pio   =0;	/* PIO unused */

	t_ac_cts_7_8.rsrv1 =0;
	t_ac_cts_7_8.rsrv2 =0;
	t_ac_cts_7_8.ver   =0;  /* read-back the version value */


	t_ac_dts_5_6.lo    =MCD->lo;
	t_ac_dts_5_6.le    =MCD->le;
	t_ac_dts_5_6.he    =MCD->he;
	t_ac_dts_5_6.ro    =MCD->ro;
	t_ac_dts_5_6.se    =0;	/* speaker disable */
	t_ac_dts_5_6.adi   =0;
	

	t_ac_dts_7_8.lg    =MCD->lg;
	t_ac_dts_7_8.is    =MCD->is;
	t_ac_dts_7_8.pio   =0;
	t_ac_dts_7_8.ovr   =0;

	t_ac_dts_7_8.rg    =MCD->rg;
	t_ac_dts_7_8.ma    =MCD->ma;


	cmode_w0	=  *(unsigned int*) &t_ac_cts_1_2;
	cmode_w1	=  *(unsigned int*) &t_ac_cts_3_4;
	cmode_w2	=  *(unsigned int*) &t_ac_cts_5_6;
	cmode_w3	=  *(unsigned int*) &t_ac_cts_7_8;

	dmode_w2	=  *(unsigned int*) &t_ac_dts_5_6;
	dmode_w3	=  *(unsigned int*) &t_ac_dts_7_8;

	scan_channels = MCD->chan & 0x3;
}

/* This delay is used to ensure that the transmission procedure will
   go out of synchronization (by missing the transmitted code)
   after the 64-bit frame has been sent and received
   in the CONTROL mode.
*/
  

_delay_1()
{ int i;
	for(i=0;i<100;i++);
}

void rx_cmode_ccl()
{
	_x_wcount=1;		/* initialize R/X interrupt handlers parameters */
	_r_wcount=0;

	x_cmode=r_cmode=ML_ON;	/* restart receiver and transmitter */

        if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
           {
	     MCBSP_FSX_ENABLE;	// enable FSX generator
	   }
	  else
	   {
             *C54X_BSP_SPC &= ~BSP_SPC_GO_MASK ;
	     *C54X_BSP_SPC |=  BSP_SPC_GO_MASK ;
	   }

	C54X_BSPR_INT_CLEAR;

        if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
	    *C54X_MCBSP0_DXR1=cmode_w0;
	 else
            *C54X_BSP_DXR=cmode_w0;	/* stimulate first BSPX-interrupt */

	WAIT_CMODE_RX_OK;	/* wait for CMODE_W0/W1 frames transferred and received */

        if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
	  {
	    MCBSP_FSX_DISABLE;
	  }


	_delay_1();		/* wait 100us to ensure that all data has been transferred */

}


_delay_40ms()
{ unsigned long i;
	for(i=0L;i<100000L;i++);
}


int configure_ac()
	/* this function generates config info for AC,
	   configures AC in CONTROL mode and then
	   goes to the DATA mode with DAC-scan and ADC-scan disabled.
	*/
  { int ccl_count;

	C54X_INT_DISABLE;

        set_ac_config_data();	/* assemble AC configuration CONTROL/DATA bit slots */
	CLEAR_CLB;

	SET_AC_CMODE;    	/* set D/C control line to CONTROL mode */
	init_sio_cmode();	/* configure DSP SIO-0 port */

	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_ENABLE;   /* enable interrupts	*/
	C54X_BSPR_INT_ENABLE;

	C54X_INT_ENABLE;	/* enable CMODE transmission with CLB=0 */

	rx_cmode_ccl();		/* perform two initial C-MODE transmit/receive cycles */
	rx_cmode_ccl();

		/* check for the returned CLB bit. If not set, then
		   repeat transmission 100 times and check again.
		   100 times repeat is required for ADI AD1849 chip,
		   which has different CLB setting procedure re CS4215.
		 */
        if((cmode_rd_w0 & _cmode_CLB_mask)!=0)
	   {
	     for(ccl_count=0;ccl_count<100;ccl_count++)	rx_cmode_ccl();

		/* check for CLB again. If still set, then return
		   the HARDWARE ERROR.
		*/
    	     if((cmode_rd_w0 & _cmode_CLB_mask)!=0) 	
			return(MERR_HW_ERR);
	   }

		/* check for read-back info from AC and compare to
		   the sent info. If does not match, then return
		   the HARDWARE	ERROR.
		*/
//        if((cmode_rd_w0 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
//			return(MERR_HW_ERR);
//        if((cmode_rd_w1 & _cmode_rd_w1_mask)!=(cmode_w1 & _cmode_rd_w1_mask))
//			return(MERR_HW_ERR);
//        if((cmode_rd_w2 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
//			return(MERR_HW_ERR);
//        if((cmode_rd_w3 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))
//			return(MERR_HW_ERR);

	do
	{
   	   rx_cmode_ccl();
	}
	while(   (cmode_rd_w0 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask)
	      || (cmode_rd_w1 & _cmode_rd_w1_mask)!=(cmode_w1 & _cmode_rd_w1_mask)
	      || (cmode_rd_w2 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask)
	      || (cmode_rd_w3 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask)
             );


		/* now transmit at least 2 frames with CLB=1
		   in order to lock AC and to prepare to go to the
		   DATA mode
		*/
	SET_CLB;	

   	   rx_cmode_ccl();
   	   rx_cmode_ccl();
   	   rx_cmode_ccl();
   	   rx_cmode_ccl();


	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	init_sio_dmode();	/* configure DSP SIO-0 port */


           	// stimulate first BSPX-interrupt
		// and initialize R/X interrupt handlers parameters
        if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
	   {
             *C54X_MCBSP0_DXR1=0;
	     _x_wcount=0;	
	     _r_wcount=0;
	   }
	 else
           {
	     *C54X_BSP_DXR=0;	
	     _x_wcount=1;	
	     _r_wcount=0;
	   }


	C54X_BSPR_INT_CLEAR;
	C54X_INT_ENABLE;		/* enable D-MODE transmission */

	SET_AC_DMODE;    	/* set D/C control line to DATA mode */
	_delay_40ms();		// wait 40ms to ensure AUTOCAL off for AD1849

	return(MERR_OK);

  }

	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  { int r;

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

       	r=configure_ac();

	if(r!=MERR_OK)	/* terminate if error is set */
	  {
	    C54X_INT_DISABLE;
	    return(r);
	  }

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) DAC;
        dac_get_ptr		= (int) DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;

	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_GO);	/* wait for MCD->status==GO */
        adc_scan_flag	= ML_ON;	/* enable saving ADC -data */

	while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */

	C54X_BSPX_INT_DISABLE;
	C54X_BSPR_INT_DISABLE;

	return(MERR_OK);
  }



	/* this procedure performs cyclic ADC->DAC scanning.
	*/
int	adc_to_dac()
  {int r;

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	scan_channels = CHAN_0 | CHAN_1;	/* assume stereo scan */

       	r=configure_ac();

	if(r!=MERR_OK)	/* terminate if error is set */
	  {
	    C54X_INT_DISABLE;
	    return(r);
	  }

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) ADC;
        dac_get_ptr		= (int) ADC;
	dac_data_count		= 0;
        dac_data_count_max	= 0x10; /* use 10-words cyclic buffer */


	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	dac_scan_flag		= ML_ON;
	adc_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_ABORT);

	C54X_INT_DISABLE;
        adc_scan_flag	= ML_OFF;
	dac_scan_flag	= ML_OFF;

	C54X_BSPX_INT_DISABLE;
	C54X_BSPR_INT_DISABLE;

	return(MERR_OK);
  }



