
#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");



#define MLD_OK           0
#define MLD_PARAM_ERR    -1



//====================== T31 specific pars ==============================

#define  T31_M_ID       0x101L       // T31 Device ID
#define  T31_PRG_ID     0x100L       // this program ID


unsigned  T31_SBUSW_SEG  = 0xd800;     // T31 SBUS Window parameters
#define  T31_SBUSW_LEN  0x8000
#define  T31_SRAM_LEN   0x200000L  // total SRAM length


#define  T31_PMAP_LSB   0x350
#define  T31_PMAP_MSB   (T31_PMAP_LSB+1) 
#define  T31_CNTR       (T31_PMAP_LSB+2)
#define  T31_BIT_CNTR   (T31_PMAP_LSB+3)

#define  T31_RESET_DATA         0
#define  T31_GO_DATA            1
#define  T31_SBUS_ERR_CLR_DATA  0x30
#define  T31_M_ATTN_DATA        0x10


#define  MCLOCK         20000000.00  // master CLOCK freq in Hz

#define  MSTAT_RDY      0L
#define  MSTAT_BUSY     1L
#define  MSTAT_ATTN     2L
#define  MSTAT_GO       3L
#define  MSTAT_ABORT    4L

#define  MERR_OK        0L
#define  MERR_INV_CMND  1L


#define  MCMND_DAS_SCAN        0x100L    // Master specific commands (DAS-scan)
#define  MCMND_DAS_DAC_SCAN    0x101L    // DAS scan with DAC output
#define  MCMND_DAC_SINGLE_LOAD 0x102L    // single word DAC load
#define  MCMND_ADC_TO_DAC	 0x103L




struct MCD_DD            // Master Communication Descriptor
 { long m_id;            // Master ID
   long prg_id;          // Master program ID

   long status;          // Master STATUS: MSTAT_RDY/BUSY/ATTN
   long err;             // command exec error

   long cmnd;            // command to exec
   long fq;		// Fq timer counter for Master
   long epoch_len;       // epoch length
   long adc_data_saddr,  // ADC data buffer start
        dac_data_saddr;  // DAC data sptr
   long chan;		 // chan map ( bit 0/1: chan-L/R on/off)

 };




#define  MCD_SADDR      (4L*0x40L)   // MCD & MDATA SBUS addr



#define LOAD_MAP(page) outpw(T31_PMAP_LSB,page)
#define SPAGE(saddr)   ((unsigned)( (long)(saddr/T31_SBUSW_LEN)))
#define SPTR(saddr)    (MK_FP(T31_SBUSW_SEG,(unsigned)((long)(saddr%T31_SBUSW_LEN))))


#define M_RESET        outp(T31_CNTR,T31_RESET_DATA); 
#define M_GO           outp (T31_CNTR,inp(T31_CNTR)&~0x1 | T31_GO_DATA); 

#define M_BYTE_ACCESS  outp(T31_CNTR,inp(T31_CNTR)&~0x30 );
#define M_WORD_ACCESS  outp(T31_CNTR,inp(T31_CNTR)&~0x30 | 0x10);
#define M_DWORD_ACCESS outp(T31_CNTR,inp(T31_CNTR)&~0x30 | 0x20);

#define T31_SET_M_ATTN     { outp(T31_BIT_CNTR+0x400,T31_M_ATTN_DATA);outp(T31_BIT_CNTR,0);outp(T31_BIT_CNTR+0x400,0); }
#define T31_CLEAR_SBUS_ERR { outp(T31_BIT_CNTR+0x400,T31_SBUS_ERR_CLR_DATA);outp(T31_BIT_CNTR,0);outp(T31_BIT_CNTR+0x400,0); } 


int TM_sptr(long addr,unsigned *page,void **p)
     /* converts SRAM ADDR to *PAGE mapper page & PC RAM **P pointer.
     */
  {  *page=SPAGE(addr);
     *p=SPTR(addr); 
     if(addr>=T31_SRAM_LEN || addr<0l)   return(MLD_PARAM_ERR) ;
     return(MLD_OK);
   }


int TM_set_sptr(long addr,void **p)
     /* check & set SBUS MAPPER page to the specified SRAM address.
        Returns converted to the SRAM MAPPER pointer to *P.
     */
  {  int r;
     unsigned page;

     if(r=MLD_PARAM_ERR, 
        (r=TM_sptr(addr,&page,p))!=MLD_OK)
           goto exit ;

     LOAD_MAP(page);
     r=MLD_OK;

    exit:
       return(r);
   }


int SBUS_err()
   { return((inp(T31_BIT_CNTR) & 0x80 )==0x80? 1:0);
   }

//========================================================================


