/********************************************************************************
	T/SDAS-AU8/L Stand-alone Demo for TORNADO-54x/E54x DSP Systems/Controllers.
	(C) MicroLAB Systems Ltd, 1994-2002

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com


	Description:
	------------
	  1. This demo runs on TORNADO-54x DSP Systems and TORNADO-E54x DSP Controllers 
		 within DSP environment only in stand-alone mode and does not require 
		 host control.
	  2. This DSP demo has been designed to be user configured by means of
		 compiler run-time definitions inside AU8L_C54.CFG file, that allows to 
		 select different demo modes and parameters:
			- TORNADO platform selection (either TORNADO-54x, or TORNADO-E54x)
			- number of AD/DA analog signal acquisition channels involved (2x/4x/8x)
			- demo mode selection (either SINUS output via ll DAC channels, or
			  ADC->DAC pass-through mode for ach AD/DA channel)
			- CS4215/AD1849 audio-codec (AC) run-time parameters (sampling frequency,
			  input selection, input gain, output attenuation, etc)
			- DSP serial port type (BSP or McBSP) used for communication with AC
			  for TORNADO-54x DSP systems
	  3. MIRAGE-510DX/P510D JTAG emulator and TI C5000 CCS tools are required to 
		 recompile this demo and to upload code to the DSP environment.
	  4. It is possible to debug DSP code using JTAG emulator.
	  5. BSP/McBSP0 SIO port is used for communication with T/SDAS-AU8/L DCM. DSP 
		 software automatically recognizes the DSP serial port type for TORNADO-E54x
		 DSP controllers, however for TORNADO-54x DSP systems this must be defined
		 via compiler run-time definitions, since there is no possibility to recognize
		 DSP type from DSP software for TORNADO-54x DSP systems.
	  6. T54x/E54x XIO-1 I/O pin must be configured as OUTPUT and is used for
		 control of DATA/CONTROL mode input of CS4215/AD1849 AC chip.
	  7. T54x/E54x XIO-0 I/O pin must be configured as OUTPUT and is used as
		 optional performance tracking output.
	  8. The demo performance is as it is and results in at least 8kHz sampling frequency
     	 for all TORNADO-54x/E54x DSP systems/controllers running at 66MHz and higher 
		 frequencies. The lower DSP frequencies might result in performance degradation.


	T/SDAS-AU8L daughter-card module installation:
	-----------------------------------------------
	  1. T/SDAS-AU8L module should be installed into SIOX-A site
  	  2. J1 jumper of T/SDAS-AU8L module must be configured for XIOF-1 line control
  	  3. Configure TORNADO-54x J3 jumper to have the XIOF-0 output on the
         XIOF-0/TM pin of SIOX site for real-time performance tracking
	     (TORNADO-54x DSP Systems rev.1 only).


	Loading Demo and Initialization:
	---------------------------------
		All demos can be loaded into DSP environment via MIRAGE JTAG emulator 
		(or compatible). Note, that the OVLY bit must be set to '1' prior loading
		the program in order to ensure that program area is loaded to on-chip memory 
		for sufficient system performance.


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


	file: main .C file

*********************************************************************************/


#include	<math.h>
#include	"c54x_def.h"	// include C54x DSP definitions


// DSP master control descriptor (MCD)

struct	MCD_DD
	{ int		status,
    			err,
				cmnd;

	  unsigned 	fq;
	  unsigned 	epoch_len,
				chan;		// AD/DA chanels involved

		// CS4215/AD1849 codec specific parameters for DMODE_B4..B7 of real-time DATA mode 
	int	lo,			// left output attenuation   (0..63)
		ro,			// right output attenuation  (0..63)
		lg,			// left input gain           (0..15)
		rg,			// right input gain          (0..15)
		ma,			// monitor path attenuation  (0..15)
		le,			// line output enable        (0/1) 
		he,			// headphone output enable   (0/1) 
		is,			// input selector (0-line input, 1-mic input) 
		mcbsp_flag;	// LSB==0 for C54x with BSP port (C542/C548/C549)
					// LSB==1 for C54xx with McBSP ports (C5402/C5410/C5416)
	} ;

// default address for MCD in D-SRAM is *(0xff00)
struct	MCD_DD *MCD	= (struct MCD_DD *)0xff00;

#define	MSTAT_RDY				0 	// Master status values
#define	MSTAT_BUSY				1
#define	MSTAT_ATTN				2
#define	MSTAT_GO				3
#define	MSTAT_ABORT				4

#define	MERR_OK					0 	// Master returned error codes
#define	MERR_INV_CMND			1 	//     ; invalid command 
#define	MERR_INV_ATTN			2
#define	MERR_HW_ERR				3

#define	MCMND_NOP				0
#define	MCMND_ABORT				1

#define	MCMND_DAC_SINE_OUTPUT	0x102
#define	MCMND_ADC_TO_DAC		0x103

#define	ML_OFF					0
#define	ML_ON					1

#define	MDATA_BUF_LEN			(0x3c00/8) 	// maximum data array length


extern  unsigned int r_buf_start, x_buf_start;
int	*ADC,	*DAC;				// ptrs to the ADC/DAC data 

int	mcbsp_flag=0;				// is set to '1' for C54xx/T54xx with McBSP ports

#define	SIO_NOT_MCBSP			(mcbsp_flag==0)
#define	SIO_MCBSP				(mcbsp_flag==1)




//---------------------- include DEMO CNF file ---------------------------
#include	"au8l_c54.cfg"	// include run-time compiler defs for demo mode and params

	// perform TORNADO-E54x specific definitions and include
#if(_E54X_SELECTED)
#include	"te54x_d.h"		// include defs for E54x

		// define XIO-flags control macros
#define	_SET_XIOF1_OFF_		TE54X_SET_XIO1_DATA_OFF
#define	_SET_XIOF1_ON_		TE54X_SET_XIO1_DATA_ON
#define	_SET_XIOF0_OFF_		TE54X_SET_XIO0_DATA_OFF
#define	_SET_XIOF0_ON_		TE54X_SET_XIO0_DATA_ON
#endif

	// perform TORNADO-54x specific definitions and include
#if(_T54X_SELECTED)
#include	"t54x_dsp.h"	// include defs for T54x

		// define XIO-flags control macros
#define	_SET_XIOF1_OFF_		T54X_set_xiof1(0)
#define	_SET_XIOF1_ON_		T54X_set_xiof1(1)
#define	_SET_XIOF0_OFF_		T54X_set_xiof0(0)
#define	_SET_XIOF0_ON_		T54X_set_xiof0(1)
#endif


//-----------------------------------------------------------------------

// include T/SDAS-AU8/L application specific DSP code
#include	"au8l_c54.i"	



cpu_init()
 {
  *C54X_IMR=0;								// clear IMR since it is not cleared on reset
  *C54X_IFR=0;

	//~~~ perform platform specific initialization

		// perform TORNADO-E54x DSP System initialization
#if(_E54X_SELECTED)
	TE54X_init_for_xdm_dram();				// initialize DSP and assume that external DATA-RAM is active

  	TE54X_SET_XIO1_CNF_IO;					// configure XIO-0/1 as OUTPUT pins
	TE54X_SET_XIO0_CNF_IO;
  	TE54X_SET_XIO1_DIR_OUTPUT;			 	
  	TE54X_SET_XIO0_DIR_OUTPUT;			 	
	TE54X_SET_XIO0_DATA_OFF;
	TE54X_SET_XIO1_DATA_OFF;

  	TE54X_set_pxsx_reset_rg(0x7);			// remove SIOX/PIOX reset
#endif

		// perform TORNADO-54x DSP System initialization
#if(_T54X_SELECTED)	
  T54X_init_dsp();							// initialize DSP

			
//  T54X_set_xio0_cnf(T54X_XIO_IO_PIN); 		// configure XIO-0/1 as OUTPUT pins
//  T54X_set_xio1_cnf(T54X_XIO_IO_PIN); 		
  T54X_set_xiof1_dir(T54X_XIOF_OUTPUT);
  T54X_set_xiof0_dir(T54X_XIOF_OUTPUT); 	
  T54X_set_xiof1(0); 	
  T54X_set_xiof0(0); 	

  T54X_set_pxsx_run(0x7);					// remove SIOX/PIOX reset
#endif
 }



#define	M_PI	3.14

void fill_dac_sine()
// fill-in DAC buffers with sine data with different frequency for each channel
{
	int *D;
    unsigned  chan_count,n;
	float scan_len;

   	scan_len=(float)MDATA_BUF_LEN;

    for (chan_count=0,D=DAC; chan_count<8; chan_count++)
       {
            for(n=0;n<MDATA_BUF_LEN;n++)
			{
			 switch(chan_count)
			  {
			   case 0:
			     *D++ = (int) ((float) (10000.0*sin(50.0*M_PI*((float)n)/scan_len)));
			      break;

			   case 1:
			     *D++ = (int) ((float) (10000.0*sin(75.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 2:
			     *D++ = (int) ((float) (10000.0*sin(100.0*M_PI*((float)n)/scan_len)));
		    	 break;

			   case 3:
			     *D++ = (int) ((float) (10000.0*sin(150.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 4:
			     *D++ = (int) ((float) (10000.0*sin(200.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 5:
			     *D++ = (int) ((float) (10000.0*sin(300.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 6:
			     *D++ = (int) ((float) (10000.0*sin(400.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 7:
			     *D++ = (int) ((float) (10000.0*sin(600.0*M_PI*((float)n)/scan_len)));
			     break;
			  }
		    }
       }
}








int	cmnd_decoder()							// command decoder
 {
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
       	// application specific commands
	   case MCMND_DAC_SINE_OUTPUT:
	   	 return( das_dac_scan() );

	   case MCMND_ADC_TO_DAC:
	   	 return( adc_to_dac() );

		// all unknown commands cause error
	   default:			
		 return( MERR_INV_CMND );
	 }
 }






void main()
{
	cpu_init();

	//--- copy run-time compiler demo mode and params to the MCD descriptor

		//~~~ set mcbsp_flag i accordance with the board type
#if(_E54X_SELECTED)
	if(TE548_BOARD || TE549_BOARD)
		MCD->mcbsp_flag =  0;
	else
		MCD->mcbsp_flag =  1;
#endif

#if(_T54X_SELECTED)
#if(_T54X_BSP_PORT_SELECTED)
	MCD->mcbsp_flag =  0;
#endif
#if(_T54X_MCBSP_PORT_SELECTED)
	MCD->mcbsp_flag =  1;
#endif
#endif


		//~~~ set number of AC AD/DA channels involved into the demo
	MCD->chan = _DAQ_CHANNELS_ ;


		//~~~ set demo function mode
	MCD->cmnd = _FUNC_ ;


		//~~~ set number of data samples per channel
	MCD->epoch_len = MDATA_BUF_LEN;


		//~~~ set AC configuration parameters, which will be programmed into
		//    AC on demo run
	MCD->fq = _AC_FS_;
	MCD->lo = _AC_LO_;
	MCD->ro = _AC_RO_;
	MCD->lg = _AC_LG_;
	MCD->rg = _AC_RG_;
	MCD->ma = _AC_MA_;
	MCD->le = _AC_LE_;
	MCD->he = _AC_HE_;
	MCD->is = _AC_IS_;


	//--- perform system parameters initialization
	ADC	=   (int *) &r_buf_start ;
	DAC	=   (int *) &x_buf_start ;

		// copy McBSP port detector
	mcbsp_flag = MCD->mcbsp_flag;	

		// fill-in DAC sine array in case DAC SINE output function has been selected
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	//--- start command execution & return error code in case of any error
   	MCD->err = cmnd_decoder();		
	MCD->status = MSTAT_RDY;			

}
