/***************************************************************
	Application specific execution module.
	device: T/SDAS-AU8/L for TORNADO-54x/E54x
****************************************************************/

/*
   Control/Data time slots for CS4215 Audio-codec chip.
   Organized as 16-bit words for portability between 32-bit and 16-bit DSPs.
   The lower significant time slots should be allocated into MSB of 16-bit word.

   Declaration of TLC command data structures.

   Note:   
	  TI C54x C Compiler processes bit fields in the reverse manner re C3x/C6xxx compiler, 
	  i.e. first declared fields appear in the most significant bits of the structure
	  word. (see TI C54x C Compiler UG, section 5.1.7 (Field/Structure Alignment).
	  Note that TI C3x C Compiler provides bit fields processing in the
	  diret manner (!!!).
*/


typedef struct     				// control time slots 1 & 2
	{
	  unsigned int rsrv3  :3; 	// should be ==001 always
	  unsigned int mlb    :1;
	  unsigned int olb    :1;
	  unsigned int clb    :1;
	  unsigned int rsrv2  :2;	// CTS1: reserved bits
	  unsigned int hpf    :1;
	  unsigned int rsrv1  :1;
	  unsigned int dfr    :3;	// DFR-0..2
	  unsigned int st     :1;
      unsigned int df     :2; 	// CTS2:  DF-0/1 
	} _AC_CTS_1_2;

typedef struct     				// control time slots 3 & 4
	{
	  unsigned int its    :1;
	  unsigned int mck    :3;
	  unsigned int bsel   :2;
	  unsigned int xclk   :1;
	  unsigned int xen    :1; 	// CTS3
	  unsigned int test   :6;
	  unsigned int enl    :1;
      unsigned int dad    :1; 	// CTS4
	} _AC_CTS_3_4;

typedef struct     				// control time slots 5 & 6 
	{
	  unsigned int pio    :2;
	  unsigned int rsrv2  :6; 	// CTS5
      unsigned int rsrv1  :8; 	// CTS6 is reserved
	} _AC_CTS_5_6;

typedef struct     				// control time slots 7 & 8 
	{
	  unsigned int rsrv2  :4;
	  unsigned int ver    :4; 	// CTS7
	  unsigned int rsrv1  :8; 	// CTS8 is reserved
	} _AC_CTS_7_8;


typedef struct     				// data time slots 1 & 2 
	{ int data_l  :16; 			// DTS-1/2, left ADC/DAC data
	} _AC_DTS_1_2;

typedef struct     				// data time slots 3 & 4 
	{ int data_r  :16; 			// DTS-3/4, right ADC/DAC data 
	} _AC_DTS_3_4;

typedef struct     				// data time slots 5 & 6 
	{
	  unsigned int he    :1;
	  unsigned int le    :1;
	  unsigned int lo    :6; 	// DTS-5 
	  unsigned int adi   :1;
	  unsigned int se    :1;
      unsigned int ro    :6; 	// DTS-6, right output attenuation
	} _AC_DTS_5_6;

typedef struct     				// data time slots 7 & 8
	{
	  unsigned int pio   :2;
	  unsigned int ovr   :1;
	  unsigned int is    :1;
	  unsigned int lg    :4; 	// DTS-7
	  unsigned int ma    :4;
      unsigned int rg    :4; 	// DTS-8
	} _AC_DTS_7_8;



_AC_CTS_1_2	t_ac_cts_1_2;
_AC_CTS_3_4	t_acm_cts_3_4;
_AC_CTS_3_4	t_acs_cts_3_4;
_AC_CTS_5_6	t_ac_cts_5_6;
_AC_CTS_7_8	t_ac_cts_7_8;

unsigned   	cmode_w0,			// transmitted 16-bit control time slots
            cmode_w1_m,
            cmode_w1_s,
            cmode_w2,
            cmode_w3,
			cmode_rd_w00,		// received 16-bit control time slots 
			cmode_rd_w01,
			cmode_rd_w02,
			cmode_rd_w03,
			cmode_rd_w10,
			cmode_rd_w11,
			cmode_rd_w12,
			cmode_rd_w13,
			cmode_rd_w20,
			cmode_rd_w21,
			cmode_rd_w22,
			cmode_rd_w23,
			cmode_rd_w30,
			cmode_rd_w31,
			cmode_rd_w32,
			cmode_rd_w33;


_AC_DTS_5_6	t_ac_dts_5_6;
_AC_DTS_7_8	t_ac_dts_7_8;

unsigned 	dmode_w02,			// transmitted 16-bit data time slots
            dmode_w03,
			dmode_w12,
			dmode_w13,
			dmode_w22,
			dmode_w23,
			dmode_w32,
			dmode_w33,
            dmode_rd_w02,		// received 16-bit data time slots
			dmode_rd_w03,
            dmode_rd_w12,
			dmode_rd_w13,
            dmode_rd_w22,
			dmode_rd_w23,
            dmode_rd_w32,
			dmode_rd_w33;



#define	_cmode_rd_w0_mask	0x0c3f 	// AND mask for CMODE_W0 to mask reserved bits
#define	_cmode_rd_w1_mask	0xbf03 	// AND mask for CMODE_W1 to mask reserved bits
#define _cmode_rd_w2_mask	0x0000 	// AND mask for CMODE_W2 to mask reserved bits
#define _cmode_rd_w3_mask	0x0000 	// AND mask for CMODE_W3 to mask reserved bits


#define	_cmode_CLB_mask	0x0400 		// CLB bit mask applies to CMODE_W0


//-------- MACROS for setting CS4215/AD1849 CONTROL/DATA mode via TIMER-1 line
#define	SET_AC_CMODE		{_SET_XIOF1_OFF_;rx_mode=RX_CMODE;}
#define	SET_AC_DMODE		{_SET_XIOF1_ON_;rx_mode=RX_DMODE;}

#define CLEAR_CLB			(cmode_w0 &=~_cmode_CLB_mask);
#define SET_CLB				(cmode_w0 |= _cmode_CLB_mask);


/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

        The tracking signal is XIOF-0/TM signal from the SIOX site connector.
	This shared signal should be configured by on-board jumper J3
	as the XIOF-0 signal (T54x rev.2A and earlier).

	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to pin XIOF-0 of SIOX connector.
	(refer to TORNADO-54x documentation re SIOX signal specs).

*/

#define _TIME_TRACKING_SET_0	_SET_XIOF0_OFF_
#define _TIME_TRACKING_SET_1	_SET_XIOF0_ON_



// X/R interrupt handler parameters

unsigned int dac_scan_flag		= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_start_ptr,
        dac_get_ptr,
		dac_data_count,
        dac_data_count_max,
		x_cmode			= 0;


unsigned int adc_scan_flag		= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_save_ptr,
		adc_data_start_ptr,
		adc_data_count,
		adc_data_count_max,
		r_cmode			= 0;


int	rx_mode	= 0;    		// C-MODE or D-MODE
int	scan_channels=0;		// number of channels for SCAN
int	fram_len=0;				// define .bsel feild vs. channels for SCAN
int	num_frame=4;			// (number of 16-bit word per frame)-1 (used by T/R ISRs)


#define	RX_CMODE		0
#define RX_DMODE		1

#define CHAN_0			1		// scanned channel IDs: left-channel AC #0
#define CHAN_1			2		// left  channel AC #0
#define CHAN_2			4		// right channel AC #1
#define CHAN_3			8		// left  channel AC #1
#define CHAN_4			0x10	// right channel AC #2
#define CHAN_5			0x20	// left  channel AC #2
#define CHAN_6			0x40	// right channel AC #3
#define CHAN_7			0x80	// left  channel AC #3

	// defs for different AD/DA channels configurations
#define DAQ_CH8_MODE   		(num_frame==15)
#define DAQ_CH4_MODE   		(num_frame==3)
#define DAQ_CH2_MODE	   	(num_frame==1)
#define DAQ_CH4_OR_CH8_MODE	(num_frame>=3)




#define WAIT_CMODE_RX_OK	while(x_cmode!=0 || r_cmode!=0);



//----------- transmitter interrupt --------------

int 		_x_wcount=0;	// transmitter word counter 
unsigned	dxr;			// data to be transmitted

interrupt void c_bsp_xint()   
// BSP transmitter interrupt 
{
//   _TIME_TRACKING_SET_1;	// set PERFORMACE TRACKING FLAG

   if(rx_mode==RX_CMODE)	// CONTROL MODE 
   {
     if(x_cmode==ML_ON)
	 {
	  switch(_x_wcount)     // decode word # and transmit the word
	   {
	     case 0:
	     case 4:
	     case 8:
	     case 12:
		  dxr=cmode_w0;
		  break;

         case 1:
		  dxr=cmode_w1_m;
		  break;

         case 2:
         case 6:
         case 10:
         case 14:
		  dxr=cmode_w2;
		  break;

         case 3:
         case 7:
         case 11:
         case 15:
		  dxr=cmode_w3;
		  break;

         case 5:
         case 9:
         case 13:
		  dxr=cmode_w1_s;
		  break;
	   }

		// load DXR/DXR1 upon the DSP type
		// Note, that no data transmission must be done if X_CMODE==ML_OFF.
		// This is used to stop Tx IRQ generation after last transmission cycle.
	   if(SIO_MCBSP)	
			   	*C54X_MCBSP0_DXR1=dxr;
		else	*C54X_BSP_DXR=dxr;


		// check for cmode transmission is complete. 
		// If YES, then clear X_CMODE flag for background control.
	   if(num_frame==_x_wcount)
	    	x_cmode=ML_OFF;   
	 }
   }
   else			
	   {	// DATA mode
	  switch(_x_wcount)     	// decode word # and transmit the word
	   {
	     case 0:				// transmit DAC channel #0 
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0) ? *((int*)dac_get_ptr):0;
		  break;

         case 1:				// transmit DAC channel #1
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0) ? *((int*)(dac_get_ptr+dac_data_count_max)):0;
		  break;

         case 2:				// transmit DMODE_W02 
		  dxr=dmode_w02;
		  break;

         case 3:				// transmit DMODE_W03
		  dxr=dmode_w03;
		  break;

	     case 4:				// transmit DAC channel #2
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_2)!=0) ? *((int*)(dac_get_ptr+2*dac_data_count_max)):0;
		  break;

         case 5:				// transmit DAC channel #3 
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_3)!=0) ? *((int*)(dac_get_ptr+3*dac_data_count_max)):0;
		  break;

         case 6:				// transmit DMODE_W12
		  dxr=dmode_w12;
		  break;

         case 7:				// transmit DMODE_W13
		  dxr=dmode_w13;
		  break;

	     case 8:				// transmit DAC channel #4
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_4)!=0) ? *((int*)(dac_get_ptr+4*dac_data_count_max)):0;
		  break;

         case 9:				// transmit DAC channel #5
		  dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_5)!=0) ? *((int*)(dac_get_ptr+5*dac_data_count_max)):0;
		  break;

         case 10:				// transmit DMODE_W22 
		  dxr=dmode_w22;
		  break;

         case 11:				// transmit DMODE_W23
		  dxr=dmode_w23;
		  break;

	     case 12:				// transmit DAC channel #6
		  dxr=((dac_scan_flag==ML_ON) && (scan_channels & CHAN_6)!=0) ? *((int*)(dac_get_ptr+6*dac_data_count_max)):0;
		  break;

         case 13:				// transmit DAC channel #7
		  dxr=((dac_scan_flag==ML_ON) && (scan_channels & CHAN_7)!=0) ? *((int*)(dac_get_ptr+7*dac_data_count_max)):0;
		  break;

         case 14:				// transmit DMODE_W32
		  dxr=dmode_w32;
		  break;

         case 15:				// transmit DMODE_W33
		  dxr=dmode_w33;
		  break;
	   }

		// load DXR/DXR1 upon the DSP type
	   if(SIO_MCBSP)	
			   	*C54X_MCBSP0_DXR1=dxr;
		else	*C54X_BSP_DXR=dxr;


		// check for update pointers and counters during last Tx-cycle in data mode
	   if(num_frame==_x_wcount)
	     {	
			if(dac_scan_flag==ML_ON)
                { dac_get_ptr++;
                  if(++dac_data_count >=dac_data_count_max)
       	            { if(dac_cyclic_scan_flag==ML_ON)
				      {
						dac_data_count=0;
                        dac_get_ptr=dac_data_start_ptr;
		      		  }
				      else
				        dac_scan_flag=ML_OFF;
		     		}
				}
	     }
   }


	// increment transmitter word counter 
    _x_wcount++;		
	_x_wcount %=(num_frame+1);

//   _TIME_TRACKING_SET_0;        // clear PERFORMANCE TRACKING FLAG

}


//----------- receiver interrupt --------------

int _r_wcount=0;				// receiver word counter 
int _r_data;

interrupt void c_bsp_rint()   
// receiver interrupt 
{
//   _TIME_TRACKING_SET_1;		// set PERFORMACE TRACKING FLAG

   _r_data= ((SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR)) ;		// read receiver data


   if(rx_mode==RX_CMODE)		
   {
	// CONTROL MODE
    if(r_cmode==ML_ON)
	 {
	  switch(_r_wcount)     	// decode word # and save as readen control word
	   {
	     case 0:
			cmode_rd_w00=_r_data;
			break;

	     case 1:
			cmode_rd_w01=_r_data;
			break;

	     case 2:
			cmode_rd_w02=_r_data;
			break;

	     case 3:
			cmode_rd_w03=_r_data;
			break;

	     case 4:
			cmode_rd_w10=_r_data;
			break;

	     case 5:
			cmode_rd_w11=_r_data;
			break;

	     case 6:
			cmode_rd_w12=_r_data;
			break;

	     case 7:
			cmode_rd_w13=_r_data;
			break;

	     case 8:
			cmode_rd_w20=_r_data;
			break;

	     case 9:
			cmode_rd_w21=_r_data;
			break;

	     case 10:
			cmode_rd_w22=_r_data;
			break;

	     case 11:
			cmode_rd_w23=_r_data;
			break;

	     case 12:
			cmode_rd_w30=_r_data;
			break;

	     case 13:
			cmode_rd_w31=_r_data;
			break;

	     case 14:
			cmode_rd_w32=_r_data;
			break;

	     case 15:
			cmode_rd_w33=_r_data;
			break;
	   }

	   if(num_frame==_r_wcount)
		  r_cmode=ML_OFF;			// clear R_CMODE flag for background control
	 }
    }
    else			
    {
		// DATA mode 
	  switch(_r_wcount)     		// decode word # and save the word 
	   {
	     case 0:					// save as ADC channel #0
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   *((int *)adc_save_ptr) = _r_data;
	      break;

	     case 1:					// save as ADC channel #1
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   *((int *)(adc_save_ptr+adc_data_count_max)) = _r_data;
	       break;

	     case 2:					// save as dmode_rd_w02
	       dmode_rd_w02 = _r_data;
	       break;

	     case 3:					// save as DMODE_RD_W03
	       dmode_rd_w03 = _r_data;
	       break;

	     case 4:					// save as ADC channel #2
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   *((int *)(adc_save_ptr+2*adc_data_count_max)) = _r_data;
	      break;

	     case 5:					// save as ADC channel #3
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   *((int *)(adc_save_ptr+3*adc_data_count_max)) = _r_data;
	       break;

	     case 6:					// save as dmode_rd_w12
	       dmode_rd_w12 = _r_data;
	       break;

	     case 7:					// save as DMODE_RD_W13
	       dmode_rd_w13 = _r_data;
	       break;

	     case 8:					// save as ADC channel #4
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   *((int *)(adc_save_ptr+4*adc_data_count_max)) = _r_data;
	      break;

	     case 9:					// save as ADC channel #5
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   *((int *)(adc_save_ptr+5*adc_data_count_max)) = _r_data;
	       break;

	     case 10:					// save as dmode_rd_w22
	       dmode_rd_w22 = _r_data;
	       break;

	     case 11:					// save as DMODE_RD_W23
	       dmode_rd_w23 = _r_data;
	       break;

	     case 12:					// save as ADC channel #6
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   *((int *)(adc_save_ptr+6*adc_data_count_max)) = _r_data;
	      break;

	     case 13:					// save as ADC channel #7 
	      if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   *((int *)(adc_save_ptr+7*adc_data_count_max)) = _r_data;
	       break;

	     case 14:					// save as dmode_rd_w32 
	       dmode_rd_w32 = _r_data;
	       break;

	     case 15:					// save as DMODE_RD_W33
	       dmode_rd_w33 = _r_data;
	       break;
	   }

		// update pointers and counters 
	   if((num_frame==_r_wcount) && (adc_scan_flag==ML_ON))
        { 
		    adc_save_ptr++;

            if(++adc_data_count >=adc_data_count_max)
		    { 
			  if(adc_cyclic_scan_flag==ML_ON)
		      {
				adc_data_count=0;
                adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		    }
		}
    }

    _r_wcount++;					// increment receiver word counter
	_r_wcount %=(num_frame+1);

//   _TIME_TRACKING_SET_0;        	// clear PERFORMACE TRACKING FLAG
}



//>>>>>>>>>> config data for SIO port <<<<<<<<<<<<<

#define	BSP_SPC_GO_MASK		0x00c0


/*----- SIO config for CONTROL MODE -----------
  BSP port (C542/C548/C549):
	FSX=out XCLK_src=int RCLK_src=ext
	XFSM=RFSM=contin  CLKRP=noram
	XLEN=RLEN=16
	DIV=31-> CLKX=CLKOUT/32


  McBSP port (C5402/C5410/C5416):
	FSX=out XCLK=int FSR=ext RCLK=ext
	XLEN=RLEN=16  XINT=RINT=enable
	number of R/X 16-bit words/frame: 4x/8x/16x (defined by the number of enabled AD/DA channels)
	serial_clock_period = CPU_CLK_period*48
	frame_sync_period   = serial_clock_period*1024
		note!!! FSYNC period is not important, since transmission is stopped after
		        full frame is transmitted. FSYNC period must be larger than max
				time transmission time.
	
*/

	// BSP port config
#define	BSP_SPC_CMODE				0x0030
#define	BSP_SPCE_CMODE				0x011F


	// McBSP port config
#define	MCBSP_SPCR1_CMODE_GO_MASK	0x0001
#define	MCBSP_SPCR2_CMODE_GO_MASK	0x0041
#define	MCBSP_SPCR1_GO_FRAME_MASK	0x0000
#define	MCBSP_SPCR2_GO_FRAME_MASK	0x0080

#define	MCBSP_SPCR1_CMODE			0x0000
#define	MCBSP_SPCR2_CMODE			0x0000

#define	MCBSP_PCR_CMODE				0x0a00

#define	MCBSP_XCR1_CH2_CMODE		0x0340	// XCR1 for 2x AD/DA channels
#define	MCBSP_XCR1_CH4_CMODE		0x0740	// XCR1 for 4x AD/DA channels
#define	MCBSP_XCR1_CH8_CMODE		0x0f40	// XCR1 for 8x AD/DA channels
#define	MCBSP_XCR2_CMODE			0x0005
#define	MCBSP_RCR1_CH2_CMODE		0x0340	// RCR1 for 2x AD/DA channels
#define	MCBSP_RCR1_CH4_CMODE		0x0740	// RCR1 for 4x AD/DA channels
#define	MCBSP_RCR1_CH8_CMODE		0x0f40	// RCR1 for 8x AD/DA channels
#define	MCBSP_RCR2_CMODE			0x0005

#define	MCBSP_SRGR1_CMODE			0x0030	// CLKGDV=48, FSPER=1024
#define	MCBSP_SRGR2_CMODE			0x3400


	// macros to enable/disable FSG during CMODE transmission
#define MCBSP_FSX_ENABLE		{*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;*C54X_MCBSP0_SPCR2 |= MCBSP_SPCR2_GO_FRAME_MASK;}
#define	MCBSP_FSX_DISABLE		{*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;*C54X_MCBSP0_SPCR2 &= ~MCBSP_SPCR2_GO_FRAME_MASK;}



_delay_2()
// this delay is used in order to ensure that BSP/McBSP port 
// has been synchronized properly after configuration complete 
{ int i;
	for(i=0;i<100;i++);
}


void init_sio_cmode()
// Configures DSP on-chip serial port for codec control-mode.
// DSP on-chip serial port is configured upon the port type (BSP/McBSP).
 {
   if(SIO_MCBSP)	
    {
 		//-- configure C54xx McBSP-0 SIO port
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = MCBSP_PCR_CMODE;

			// RCR1 requires different data for different frame formats
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = ((DAQ_CH8_MODE)? MCBSP_RCR1_CH8_CMODE:
								((DAQ_CH4_MODE)? MCBSP_RCR1_CH4_CMODE:MCBSP_RCR1_CH2_CMODE)
							);
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = MCBSP_RCR2_CMODE;

			// XCR1 requires different data for different frame formats
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = ((DAQ_CH8_MODE)? MCBSP_XCR1_CH8_CMODE:
								((DAQ_CH4_MODE)? MCBSP_XCR1_CH4_CMODE:MCBSP_XCR1_CH2_CMODE)
							);

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = MCBSP_XCR2_CMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP0_SRGR1 = MCBSP_SRGR1_CMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP0_SRGR2 = MCBSP_SRGR2_CMODE;

		_delay_2();

		// now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_CMODE_GO_MASK;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_CMODE_GO_MASK;

		_delay_2();
    }


	else
	{
		//-- configure C548/C549/C542 DSP with BSP/TDM ports
	   *C54X_BSP_SPC  = BSP_SPC_CMODE;						// reset BSP
	   *C54X_BSP_SPCE = BSP_SPCE_CMODE;

		_delay_2();

	   *C54X_BSP_SPC  = BSP_SPC_CMODE | BSP_SPC_GO_MASK;	// enable BSP

		_delay_2();
	}

 }




/*----- SIO config for DATA MODE ---------
  BSP port (C542/C548/C549):
	FSX=out XCLK_src=RCLK_src=ext
	XFSM=RFSM=contin
	XLEN=RLEN=16


  McBSP port (C5402/C5410/C5416):
	FSX=ext XCLK=ext FSR=ext RCLK=ext
	XLEN=RLEN=16  XINT=RINT=enable
	number of R/X 16-bit words/frame: 4x/8x/16x (defined by the number of enabled AD/DA channels)
*/

	// BSP port config
#define	BSP_SPC_DMODE				0x0000
#define	BSP_SPCE_DMODE				0x0100


	// McBSP port config
#define	MCBSP_SPCR1_DMODE_GO_MASK	0x0001
#define	MCBSP_SPCR2_DMODE_GO_MASK	0x0001

#define	MCBSP_SPCR1_DMODE			0x0000
#define	MCBSP_SPCR2_DMODE			0x0000
#define	MCBSP_PCR_DMODE				0x0000

#define	MCBSP_XCR1_CH2_DMODE		0x0340	// XCR1 for 2x AD/DA channels
#define	MCBSP_XCR1_CH4_DMODE		0x0740	// XCR1 for 4x AD/DA channels
#define	MCBSP_XCR1_CH8_DMODE		0x0f40	// XCR1 for 8x AD/DA channels
#define	MCBSP_XCR2_DMODE			0x0005
#define	MCBSP_RCR1_CH2_DMODE		0x0340	// RCR1 for 2x AD/DA channels
#define	MCBSP_RCR1_CH4_DMODE		0x0740	// RCR1 for 4x AD/DA channels
#define	MCBSP_RCR1_CH8_DMODE		0x0f40	// RCR1 for 8x AD/DA channels
#define	MCBSP_RCR2_DMODE			0x0005

#define	MCBSP_SRGR1_DMODE			0x0000
#define	MCBSP_SRGR2_DMODE			0x0000



void init_sio_dmode()
// Configures DSP on-chip serial port for codec data-mode.
// DSP on-chip serial port is configured upon the port type (BSP/McBSP).
 {
   if(SIO_MCBSP)	
    {
 		//-- configure C54xx McBSP-0 SIO port
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = MCBSP_PCR_DMODE;

			// RCR1 requires different data for different frame formats
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = ((DAQ_CH8_MODE)? MCBSP_RCR1_CH8_DMODE:
								((DAQ_CH4_MODE)? MCBSP_RCR1_CH4_DMODE:MCBSP_RCR1_CH2_DMODE)
							);
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = MCBSP_RCR2_DMODE;

			// XCR1 requires different data for different frame formats
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = ((DAQ_CH8_MODE)? MCBSP_XCR1_CH8_DMODE:
								((DAQ_CH4_MODE)? MCBSP_XCR1_CH4_DMODE:MCBSP_XCR1_CH2_DMODE)
							);
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = MCBSP_XCR2_DMODE;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP0_SRGR1 = MCBSP_SRGR1_DMODE;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP0_SRGR2 = MCBSP_SRGR2_DMODE;

		_delay_2();

		// now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DMODE_GO_MASK;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DMODE_GO_MASK;

		_delay_2();
    }
     else	
    {
		//-- configure C548/C549/C542 DSP with BSP/TDM ports
	   *C54X_BSP_SPC  = BSP_SPC_DMODE;						// reset BSP
	   *C54X_BSP_SPCE = BSP_SPCE_DMODE;

		_delay_2();

	   *C54X_BSP_SPC  = BSP_SPC_DMODE | BSP_SPC_GO_MASK;	// enable BSP

		_delay_2();
	}
 }






void set_ac_config_data()
{
	scan_channels	= MCD->chan & 0xff;

		// each channel (#0..7) has its own enable bit
		// configure run-time params in accordance with enabled channels (2x/4x/8x)

	if(scan_channels>0xf)		
	  {	// set params for 8x AD/DA channels
	    	fram_len	= 2;
	    	num_frame	= 15;
	  }
	 else
	   if(scan_channels>0x3)	
	    { // set params for 4x AD/DA channels
	      	fram_len	= 1;
	      	num_frame	= 7;
	    }
	    else
	     {// set params for 2x AD/DA channels
			fram_len	= 0;
	      	num_frame	= 3;
	     }

	// create CMODE words #0/#1 and DMODE word #1 
	t_ac_cts_1_2.clb=0;     		// CLB bit =0 
	t_ac_cts_1_2.olb=0;				// select 1Vrms scale
	t_ac_cts_1_2.mlb=0;				// enable MIC +20dB amplifier

    t_ac_cts_1_2.rsrv3=1;			// reserved bits 
	t_ac_cts_1_2.rsrv2=0;
	t_ac_cts_1_2.rsrv1=0;

	t_ac_cts_1_2.df = 0;			// 16-bit data
	t_ac_cts_1_2.st	= 1;			// stereo mode
	t_ac_cts_1_2.hpf = 0;			// disable high-pass filter

	// set CLK selection 
	t_ac_cts_1_2.dfr = MCD->fq & 0x7;	
	t_acm_cts_3_4.mck = (((MCD->fq)>>3) & 0x1) +1;

	t_acm_cts_3_4.xen = 0;
	t_acm_cts_3_4.xclk= 1;
	t_acm_cts_3_4.bsel= fram_len;
	t_acm_cts_3_4.its = 1;
	t_acm_cts_3_4.dad = 0;			// disable loopback and tests
	t_acm_cts_3_4.enl = 0;
	t_acm_cts_3_4.test= 0;

	t_acs_cts_3_4.mck = 3;

	t_acs_cts_3_4.xen = 0;
	t_acs_cts_3_4.xclk= 0;
	t_acs_cts_3_4.bsel= fram_len;
	t_acs_cts_3_4.its = 1;
	t_acs_cts_3_4.dad = 0;			// disable loopback and tests
	t_acs_cts_3_4.enl = 0;
	t_acs_cts_3_4.test= 0;

	t_ac_cts_5_6.rsrv1 =0;
	t_ac_cts_5_6.rsrv2 =0;
	t_ac_cts_5_6.pio   =0;			// PIO unused

	t_ac_cts_7_8.rsrv1 =0;
	t_ac_cts_7_8.rsrv2 =0;
	t_ac_cts_7_8.ver   =0;  		// read-back the version value

	t_ac_dts_5_6.lo    =MCD->lo;
	t_ac_dts_5_6.le    =MCD->le;
	t_ac_dts_5_6.he    =MCD->he;
	t_ac_dts_5_6.ro    =MCD->ro;
	t_ac_dts_5_6.se    =0;			// speaker disable 
	t_ac_dts_5_6.adi   =0;

	t_ac_dts_7_8.lg    =MCD->lg;
	t_ac_dts_7_8.is    =MCD->is;
	t_ac_dts_7_8.pio   =0;
	t_ac_dts_7_8.ovr   =0;

	t_ac_dts_7_8.rg    =MCD->rg;
	t_ac_dts_7_8.ma    =MCD->ma;

	cmode_w0	=  *(unsigned int*) &t_ac_cts_1_2;

	cmode_w1_m	=  *(unsigned int*) &t_acm_cts_3_4;
	cmode_w1_s	=  *(unsigned int*) &t_acs_cts_3_4;
	cmode_w2	=  *(unsigned int*) &t_ac_cts_5_6;
	cmode_w3	=  *(unsigned int*) &t_ac_cts_7_8;

	dmode_w02	=  *(unsigned int*) &t_ac_dts_5_6;
	dmode_w03	=  *(unsigned int*) &t_ac_dts_7_8;
	dmode_w12	=  dmode_w02;
	dmode_w13	=  dmode_w03;
	dmode_w22	=  dmode_w02;
	dmode_w23	=  dmode_w03;
	dmode_w32	=  dmode_w02;
	dmode_w33	=  dmode_w03;


}



_delay_1()
// This delay is used to ensure that the transmission procedure will
// go out of synchronization (by missing the transmitted code)
// for C542/C548/C549 DSPs with BCP port in continuous mode
// after the 64-bit frame has been sent and received in the CONTROL mode.
{ int i;
	for(i=0;i<1000;i++);
}



void rx_cmode_ccl()
{
	_x_wcount=1;				// initialize R/X interrupt handlers parameters
	_r_wcount=0;

	x_cmode=r_cmode=ML_ON;		// restart receiver and transmitter ISRs

	C54X_BSPR_INT_CLEAR;

	// BSP:
	//   - start BSP operation,
	//	 - load 1st codec word to DXR1 in order to stimulate first Tx-interrupt
	// McBSP:
	// 	 - load 1st codec word to DXR in order to stimulate first Tx-interrupt
	// 	 - enable FSX generator for McBSP and 
    if(SIO_MCBSP)		
	{
	    *C54X_MCBSP0_DXR1=cmode_w0;
		_delay_2();
	    MCBSP_FSX_ENABLE;						
	}
	else
	{
        *C54X_BSP_SPC &= ~BSP_SPC_GO_MASK ;
	    *C54X_BSP_SPC |=  BSP_SPC_GO_MASK ;
		_delay_2();
        *C54X_BSP_DXR=cmode_w0;		
	}


	// wait for CMODE_W0/W1 frames transferred and received
	WAIT_CMODE_RX_OK;				

	// Disable FSG operation for McBSP ports in order to exclude secondary frame transfer.
	// Note, that programmed FSG period must be large enough in order not to miss 
	// this operation.
    if(SIO_MCBSP)					
	{
	    MCBSP_FSX_DISABLE;
	}

	// wait 100us for BSP ports to ensure that all data has been transferred
	_delay_1();						

}



_delay_40ms()
// used to ensure that AD1849 has passed over the autocalibration procedure 
// after codec data mode has been set
{ unsigned long i;
	for(i=0L;i<100000L;i++);
}



int configure_ac()
	/* this fucnction generates config info for AC,
	   configures AC in CONTROL mode and then
	   goes to the DATA mode with DAC-scan and ADC-scan disabled.
	*/
  { 
	int	ccl_count;

	C54X_INT_DISABLE;

	// assemble AC configuration CONTROL/DATA bit slots
    set_ac_config_data();		

	// set CLB=0 for AC
	CLEAR_CLB;

	// set D/C control line to CONTROL mode
	SET_AC_CMODE;    			

	// configure DSP SIO-0 port
	init_sio_cmode();			

	// clear and enable DSP interrupts
	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_CLEAR;
	C54X_BSPX_INT_ENABLE;   	
	C54X_BSPR_INT_ENABLE;
	C54X_INT_ENABLE;			

	// perform two initial C-MODE transmit/receive cycles
	rx_cmode_ccl();				
	rx_cmode_ccl();


	// check for the returned CLB bit. If not set, then
	// repeat transmission max 100 times.
	// (This meets specs for CS4215 and AD1849 ACs).
    if(   ((cmode_rd_w00 & _cmode_CLB_mask)!=0)
	   || (DAQ_CH4_OR_CH8_MODE && ((cmode_rd_w10 & _cmode_CLB_mask)!=0))
	   || (DAQ_CH8_MODE        && ((cmode_rd_w20 & _cmode_CLB_mask)!=0))
	   || (DAQ_CH8_MODE        && ((cmode_rd_w30 & _cmode_CLB_mask)!=0))
	  )
	   {
	     for(ccl_count=0;ccl_count<100;ccl_count++)	rx_cmode_ccl();

		// check for CLB again. If still set, then return the HARDWARE ERROR.
  	     if(   ((cmode_rd_w00 & _cmode_CLB_mask)!=0)
		   	|| (DAQ_CH4_OR_CH8_MODE && ((cmode_rd_w10 & _cmode_CLB_mask)!=0))
		   	|| (DAQ_CH8_MODE        && ((cmode_rd_w20 & _cmode_CLB_mask)!=0))
	   		|| (DAQ_CH8_MODE        && ((cmode_rd_w30 & _cmode_CLB_mask)!=0))
	       )
			return(MERR_HW_ERR);
	   }

	// continuously write control info to AC and read-back until
	// readen info will match the written one.
	do
	{
   	   rx_cmode_ccl();
	}
	while(   ((cmode_rd_w00 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      || ((cmode_rd_w01 & _cmode_rd_w1_mask)!=(cmode_w1_m & _cmode_rd_w1_mask))
	      || ((cmode_rd_w02 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      || ((cmode_rd_w03 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))

		  || (DAQ_CH4_OR_CH8_MODE && 
	      	   (   ((cmode_rd_w10 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      		|| ((cmode_rd_w11 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w12 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w13 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))
			   )
			 )

		  || (DAQ_CH8_MODE && 
		       (   ((cmode_rd_w20 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
				|| ((cmode_rd_w21 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w22 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w23 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))

	      		|| ((cmode_rd_w30 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      		|| ((cmode_rd_w31 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w32 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w33 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))
			   )
			 )
       );

	// now transmit at least 2 frames with CLB=1
	// in order to lock AC and to prepare to go to the DATA mode
	SET_CLB;

    rx_cmode_ccl();
    rx_cmode_ccl();
    rx_cmode_ccl();
    rx_cmode_ccl();


	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	// now configure DSP SIO-0 port for AC data mode
	init_sio_dmode();		
	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_CLEAR;

	// initialize R/X interrupt handlers parameters
     _x_wcount=1;	
     _r_wcount=0;

   	// stimulate first BSPX-interrupt (load DXR/DXR1 upon the DSP type)
    if(SIO_MCBSP)	
         *C54X_MCBSP0_DXR1=0;
	 else
	     *C54X_BSP_DXR=0;	

	// wait some time to have DX loaded and all BSP synchronization setup
	_delay_2();


	// enable AC D-MODE transmission 
	C54X_INT_ENABLE;		

	// set D/C control line to DATA mode 
	SET_AC_DMODE;    		

	// wait 40ms to ensure AUTOCAL off for AD1849
	_delay_40ms();			

	return(MERR_OK);

  }



int	das_dac_scan()
// this procedure performs cyclic DAC output.
  { int r;

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

   	r=configure_ac();

	// terminate if error is set 
	if(r!=MERR_OK)	
	  {
	    C54X_INT_DISABLE;
	    return(r);
	  }

	// set X/R interrupt driver parameters
	dac_data_start_ptr		= (int) DAC;
    dac_get_ptr				= (int) DAC;
	dac_data_count			= 0;
    dac_data_count_max		= MCD->epoch_len;

	adc_data_start_ptr		= (int) ADC;
    adc_save_ptr			= (int) ADC;
	adc_data_count			= 0;
    adc_data_count_max		= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag			= ML_OFF;
	dac_scan_flag			= ML_ON;

	// loop forever
	while(1);
  }


int	adc_to_dac()
// this procedure performs cyclic ADC->DAC data copy (pass-through channel)
  {int r;

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

   	r=configure_ac();

	// terminate if error is set 
	if(r!=MERR_OK)	
	  {
	    C54X_INT_DISABLE;
	    return(r);
	  }

	// set X/R interrupt driver parameters
	dac_data_start_ptr		= (int) ADC;
    dac_get_ptr				= (int) ADC;
	dac_data_count			= 0;
    dac_data_count_max		= 0x10; 	// use 10-words cyclic buffer for ADC->DAC copy


	adc_data_start_ptr	= (int) ADC;
    adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
    adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	dac_scan_flag		= ML_ON;
	adc_scan_flag		= ML_ON;

	// loop forever
	while(1);
  }
