/***************************************************************
	Application specific execution module.
	device: T/SDAS-AU8L
****************************************************************/

/*
   Control/Data time slots for CS4215 Audio-codec chip.
   Organized as 16-bit words for portability between 32-bit and 16-bit DSPs.
   The lower significant time slots should be allocated into MSB of 16-bit word.
*/

typedef struct     	/* control time slots 1 & 2 */
	{ unsigned int df     :2; /* CTS2:  DF-0/1 */
	  unsigned int st     :1;
	  unsigned int dfr    :3; /*        DFR-0..2*/
	  unsigned int rsrv1  :1;
	  unsigned int hpf    :1;

	  unsigned int rsrv2  :2; /* CTS1: reserved bits */
	  unsigned int clb    :1;
	  unsigned int olb    :1;
	  unsigned int mlb    :1;
	  unsigned int rsrv3  :3; /* should be ==001 always */
	} _AC_CTS_1_2;

typedef struct     	/* control time slots 3 & 4 */
	{ unsigned int dad    :1; /* CTS4*/
	  unsigned int enl    :1;
	  unsigned int test   :6;

	  unsigned int xen    :1; /* CTS3 */
	  unsigned int xclk   :1;
	  unsigned int bsel   :2;
	  unsigned int mck    :3;
	  unsigned int its    :1;
	} _AC_CTS_3_4;

typedef struct     	/* control time slots 5 & 6 */
	{ unsigned int rsrv1  :8; /* CTS6 is reserved*/

	  unsigned int rsrv2  :6; /* CTS5*/
	  unsigned int pio    :2;
	} _AC_CTS_5_6;

typedef struct     	/* control time slots 7 & 8 */
	{ unsigned int rsrv1  :8; /* CTS8 is reserved*/

	  unsigned int ver    :4; /* CTS7*/
	  unsigned int rsrv2  :4;
	} _AC_CTS_7_8;

typedef struct     	/* data time slots 1 & 2 */
	{ int data_l  :16; /* DTS-1/2, left ADC/DAC data */
	} _AC_DTS_1_2;

typedef struct     	/* data time slots 3 & 4 */
	{ int data_r  :16; /* DTS-3/4, right ADC/DAC data */
	} _AC_DTS_3_4;

typedef struct     	/* data time slots 5 & 6 */
	{ unsigned int ro    :6; /* DTS-6, right output attenuation */
	  unsigned int se    :1;
	  unsigned int adi   :1;

	  unsigned int lo    :6; /* DTS-5 */
	  unsigned int le    :1;
	  unsigned int he    :1;
	} _AC_DTS_5_6;

typedef struct     	/* data time slots 7 & 8 */
	{ unsigned int rg    :4; /* DTS-8 */
	  unsigned int ma    :4;

	  unsigned int lg    :4; /* DTS-7 */
	  unsigned int is    :1;
	  unsigned int ovr   :1;
	  unsigned int pio   :2;
	} _AC_DTS_7_8;


_AC_CTS_1_2	t_ac_cts_1_2;
_AC_CTS_3_4	t_acm_cts_3_4,t_acs_cts_3_4;
_AC_CTS_5_6	t_ac_cts_5_6;
_AC_CTS_7_8	t_ac_cts_7_8;

unsigned int    cmode_w00,cmode_w10,/* transmitted 32-bit control time slots */
                cmode_w1,cmode_w11,
		cmode_rd_w00,cmode_rd_w01,/* received 32-bit control time slots */
		cmode_rd_w10,cmode_rd_w11,
		cmode_rd_w20,cmode_rd_w21,
		cmode_rd_w30,cmode_rd_w31;

_AC_DTS_5_6	t_ac_dts_5_6;
_AC_DTS_7_8	t_ac_dts_7_8;

unsigned int    dmode_w01,	/* transmitted 32-bit data time slots */
		dmode_w11,
                dmode_rd_w01,	/* received 32-bit data time slots */
		dmode_rd_w11,
		dmode_rd_w21,
		dmode_rd_w31;

#define	_cmode_rd_w0_mask	0x0c3fbf03 /* AND mask for CMODE_W0 to mask reserved bits */
#define _cmode_rd_w1_mask	0x00000000 /* AND mask for CMODE_W1 to mask reserved bits */

#define	_cmode_CLB_mask	0x04000000 /* CLB bit mask */

/*-------- MACROS for setting CS4215 CONTROL/DATA mode via TIMER-1 line */
#define	SET_AC_CMODE	{*TM1_GC=2;rx_mode=RX_CMODE;}
#define	SET_AC_DMODE	{*TM1_GC=6;rx_mode=RX_DMODE;}

#define CLEAR_CLB	{(cmode_w00 &=~_cmode_CLB_mask);(cmode_w10 &=~_cmode_CLB_mask);}
#define SET_CLB		{(cmode_w00 |= _cmode_CLB_mask);(cmode_w10 |= _cmode_CLB_mask);}

/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

        The tracking signal is DSP' TIMER-0 signal from the SIOX site.
	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to pin TIMER-0 of SIOX connector.
	(refer to TORNADO-3x documentation re SIOX signal specs).
*/

#define _TIME_TRACKING_SET_0	{*TM0_GC=6;}
#define _TIME_TRACKING_SET_1	{*TM0_GC=2;}

		/* X/R interrupt handler parameters	*/

int	dac_scan_flag		= ML_OFF,
	dac_cyclic_scan_flag	= ML_ON,
	dac_data_start_ptr,
        dac_get_ptr,
	dac_data_count,
        dac_data_count_max,
	x_cmode			= 0;


int	adc_scan_flag		= ML_OFF,
	adc_cyclic_scan_flag	= ML_OFF,
	adc_save_ptr,
	adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max,
	r_cmode			= 0;


int	rx_mode	= 0;    	/* C-MODE or D-MODE */
int	scan_channels=0;	/* channels for SCAN */
int	fram_len=0;		/* define .bsel feild vs. channels for SCAN */
int	num_fram;		/* define number of frames in drivers */
unsigned int	chip0_en=0,
		chip1_en=0,
		chip2_en=0,
		chip3_en=0;

#define	RX_CMODE	0
#define RX_DMODE	1

#define CHAN_0		1	/* scanned channel IDs: AC#0: left-channel */
#define CHAN_1		2	/* AC#0: right */
#define CHAN_2		4	/* AC#1: left */
#define CHAN_3		8	/* AC#1: right */
#define CHAN_4		0x10	/* AC#2: left */
#define CHAN_5		0x20	/* AC#2: right */
#define CHAN_6		0x40	/* AC#3: left */
#define CHAN_7		0x80	/* AC#3: right */


#define WAIT_CMODE_RX_OK	while(x_cmode!=0 || r_cmode!=0);

_delay_100us()
{ int i;
	for(i=0;i<100;i++);
}

_delay_40ms()
{ int i;
	for(i=0;i<40000;i++);
}


/*----------- transmitter interrupt --------------*/

int 		_x_wcount=0;	/* transmitter word counter */
unsigned int	_x_dmode_w0;	/* assembled DAC data for DATA mode */

void c_int10()
{
   _TIME_TRACKING_SET_1;	/* set PERFORMACE TRACKING FLAG */

   if(rx_mode==RX_CMODE)	/* CONTROL MODE */
      { if(x_cmode==ML_ON)
	 {
	  switch(_x_wcount)     /* decode the W32# and transmit the word */
	     {
		case 0:
	     	     *SIO0_XD=cmode_w00;
		    break;

	     	case 1:
	     	     *SIO0_XD=cmode_w1;
		    break;
	     	case 2:
	     	     *SIO0_XD=cmode_w10;
		    break;
	     	case 3:
	     	     *SIO0_XD=cmode_w1;
		    break;
	     	case 4:
	     	     *SIO0_XD=cmode_w10;
		    break;
	     	case 5:
	     	     *SIO0_XD=cmode_w1;
		    break;
	     	case 6:
	     	     *SIO0_XD=cmode_w10;
		    break;
	     	case 7:
	     	     *SIO0_XD=cmode_w1;
		    break;

	     }
	   if(num_fram==_x_wcount)
	    x_cmode=ML_OFF;   /* clear X_CMODE flag for background control */
	 }
      }
     else			/* DATA mode */
       {
	  switch(_x_wcount)
	     {
		case 0:
		     if(dac_scan_flag==ML_OFF)
        	        *SIO0_XD=0;	/* send 0 if DAC-scan is disabled */

	      	       else 		/* DAC-scan is enabled.
	      				Assemble W32 fm two channels. */
	      		{
	      		  _x_dmode_w0=0;
			  if((scan_channels & CHAN_0)!=0)	/* check and add data fm chan-0 */
		  	  _x_dmode_w0 |= (((*((int*)dac_get_ptr))<<16) & 0xffff0000);

			  if((scan_channels & CHAN_1)!=0)	/* check and add data fm chan-1 */
		  	  _x_dmode_w0 |= ((*((int*)(dac_get_ptr+dac_data_count_max))) & 0xffff);

			  *SIO0_XD=_x_dmode_w0;	/* transmit DAC data */
			}
		    break;

		case 1:
	    	     *SIO0_XD=dmode_w01;
		    break;

		case 2:
		     if(dac_scan_flag==ML_OFF)
        	        *SIO0_XD=0;  // send 0 if DAC-scan is disabled

	      	       else 	     // DAC-scan is enabled.
	      			     //	Assemble W32 fm two channels.
	      		{
	      		  _x_dmode_w0=0;
			  if((scan_channels & CHAN_2)!=0)	// check and add data fm chan-0
		  	    _x_dmode_w0 |= (((*((int*)(dac_get_ptr+(2*dac_data_count_max))))<<16) & 0xffff0000);

			  if((scan_channels & CHAN_3)!=0)	// check and add data fm chan-1
		  	    _x_dmode_w0 |= ((*((int*)(dac_get_ptr+(3*dac_data_count_max)))) & 0xffff);

			  *SIO0_XD=_x_dmode_w0;	// transmit DAC data
		        }
		    break;

		case 3:
	    	     *SIO0_XD=dmode_w11;
		     break;

		case 4:
		     if(dac_scan_flag==ML_OFF)
        	        *SIO0_XD=0;	/* send 0 if DAC-scan is disabled */

	      	       else 		/* DAC-scan is enabled.
	      				Assemble W32 fm two channels. */
	      		{
	      		  _x_dmode_w0=0;
			  if((scan_channels & CHAN_4)!=0)	// check and add data fm chan-0
		  	    _x_dmode_w0 |= (((*((int*)(dac_get_ptr+(4*dac_data_count_max))))<<16) & 0xffff0000);

			  if((scan_channels & CHAN_5)!=0)	// check and add data fm chan-1
		  	    _x_dmode_w0 |= ((*((int*)(dac_get_ptr+(5*dac_data_count_max)))) & 0xffff);


			  *SIO0_XD=_x_dmode_w0;	/* transmit DAC data */
			}
		    break;

		case 5:
	    	     *SIO0_XD=dmode_w11;
		    break;

		case 6:
		     if(dac_scan_flag==ML_OFF)
        	        *SIO0_XD=0;	/* send 0 if DAC-scan is disabled */

	      	       else 		/* DAC-scan is enabled.
	      				Assemble W32 fm two channels. */
	      		{
	      		  _x_dmode_w0=0;
			  if((scan_channels & CHAN_6)!=0)	// check and add data fm chan-0
		  	    _x_dmode_w0 |= (((*((int*)(dac_get_ptr+(6*dac_data_count_max))))<<16) & 0xffff0000);

			  if((scan_channels & CHAN_7)!=0)	// check and add data fm chan-1
		  	    _x_dmode_w0 |= ((*((int*)(dac_get_ptr+(7*dac_data_count_max)))) & 0xffff);

			  *SIO0_XD=_x_dmode_w0;	/* transmit DAC data */
			}
		    break;

		case 7:
	    	     *SIO0_XD=dmode_w11;
		    break;
     	     }
	  if((num_fram==_x_wcount) && (dac_scan_flag==ML_ON))
	    {
		    // update pointers and counters
              dac_get_ptr++;
              if(++dac_data_count >=dac_data_count_max)
        	{ if(dac_cyclic_scan_flag==ML_ON)
		     {
		      dac_data_count=0;
                      dac_get_ptr=dac_data_start_ptr;
		     }
		   else
		     dac_scan_flag=ML_OFF;
		}
	    }

       }
        _x_wcount++;		/* increment transmitter word counter */
	_x_wcount %=(num_fram+1);

   _TIME_TRACKING_SET_0;        /* clear PERFORMACE TRACKING FLAG */
}


/*----------- receiver interrupt --------------*/

int _r_wcount=0;	/* receiver word counter */
int _r_data;

void c_int11()   /* receiver interrupt */
{
//   _TIME_TRACKING_SET_1;	/* set PERFORMACE TRACKING FLAG */

   _r_data=*SIO0_RD;		/* read receiver data */


   if(rx_mode==RX_CMODE)	/* CONTROL MODE */
      {
	if(r_cmode==ML_ON)
	  {  switch(_r_wcount)
	      {
		case 0:
		     cmode_rd_w00=_r_data;
		    break;
		case 1:
		     cmode_rd_w01=_r_data;
		    break;
		case 2:
		     cmode_rd_w10=_r_data;
		    break;
		case 3:
		     cmode_rd_w11=_r_data;
		    break;
		case 4:
		     cmode_rd_w20=_r_data;
		    break;
		case 5:
		     cmode_rd_w21=_r_data;
		    break;
		case 6:
		     cmode_rd_w30=_r_data;
		    break;
		case 7:
		     cmode_rd_w31=_r_data;
		    break;
	      }
            if(num_fram==_r_wcount)
	      r_cmode=ML_OFF;	/* clear R_CMODE flag for background control */
	  }

      }
     else
       {
           switch(_r_wcount)
	      {
		case 0:
		   if(adc_scan_flag==ML_ON)
		    {
		      if((scan_channels & CHAN_0)!=0) 	// check and unpack CHAN-0
		      	  *((int *)adc_save_ptr) = _r_data>>16;

		      if((scan_channels & CHAN_1)!=0) 	// check and unpack CHAN-1, keep sign extension
		   	  *((int *)(adc_save_ptr+adc_data_count_max)) = (_r_data<<16)>>16;
		    }

		    break;

		case 1:                  // D-mode, word #1
		     dmode_rd_w01=_r_data;
		    break;

		case 2:
		   if(adc_scan_flag==ML_ON)
		    {
		      if((scan_channels & CHAN_2)!=0) 	// check and unpack CHAN-0
		      	  *((int *)(adc_save_ptr+(2*adc_data_count_max))) = _r_data>>16;

		      if((scan_channels & CHAN_3)!=0) 	// check and unpack CHAN-1, keep sign extension
		   	  *((int *)(adc_save_ptr+(3*adc_data_count_max))) = (_r_data<<16)>>16;

		    }

		    break;

		case 3:
	     	     dmode_rd_w11=_r_data;
		    break;

		case 4:
		   if(adc_scan_flag==ML_ON)
		    {
		      if((scan_channels & CHAN_4)!=0) 	// check and unpack CHAN-0
		      	  *((int *)(adc_save_ptr+(4*adc_data_count_max))) = _r_data>>16;

		      if((scan_channels & CHAN_5)!=0) 	// check and unpack CHAN-1, keep sign extension
		   	  *((int *)(adc_save_ptr+(5*adc_data_count_max))) = (_r_data<<16)>>16;

		    }

		    break;

		case 5:
	     	     dmode_rd_w21=_r_data;
		    break;
		case 6:
		   if(adc_scan_flag==ML_ON)
		    {
		      if((scan_channels & CHAN_6)!=0) 	// check and unpack CHAN-0
		      	  *((int *)(adc_save_ptr+(6*adc_data_count_max))) = _r_data>>16;

		      if((scan_channels & CHAN_7)!=0) 	// check and unpack CHAN-1, keep sign extension
		   	  *((int *)(adc_save_ptr+(7*adc_data_count_max))) = (_r_data<<16)>>16;

		    }

		    break;

		case 7:
	     	     dmode_rd_w31=_r_data;
		    break;
	      }

	  if((num_fram==_r_wcount) && (adc_scan_flag==ML_ON))
	     {
		  /* update pointers and counters */
              adc_save_ptr++;
              if(++adc_data_count >=adc_data_count_max)
		{ if(adc_cyclic_scan_flag==ML_ON)
		    {
		      adc_data_count=0;
	              adc_save_ptr=adc_data_start_ptr;
		    }
		   else
		        adc_scan_flag=ML_OFF;
		}
	     }

       }

        _r_wcount++;		/* increment receiver word counter */
	_r_wcount %=(num_fram+1);

//   _TIME_TRACKING_SET_0;        /* clear PERFORMACE TRACKING FLAG */

}

/*>>>>>>>>>> config data for SIO port <<<<<<<<<<<<<*/

/*----- SIO config for CONTROL MODE -----------
	FSX=out XCLK_src=int RCLK_src=ext
	XFSM=RFSM=contin  CLKRP=low
	XLEN=RLEN=32  XINT=RINT=enable
*/

#define	SIO_GC_CMODE		0x02bc0c44
#define	SIO_XC_CMODE		0x111
#define	SIO_RC_CMODE		0x111
#define	SIO_XTMPERIOD_CMODE	0x10004


/*----- SIO config for DATA MODE ---------
	FSX=out XCLK_src=RCLK_src=ext
	XFSM=RFSM=contin
	XLEN=RLEN=32  XINT=RINT=enable
*/

#define	SIO_GC_DMODE		0x02bc0c00
#define	SIO_XC_DMODE		0x111
#define	SIO_RC_DMODE		0x111


#define	SIO_TMCNTR_STOP		0xC
#define	SIO_TMCNTR_GO		0xF
#define	SIO_GC_GO_MASK		0x0c000000






void init_sio_dmode()
 {
	*SIO0_GC = SIO_GC_DMODE  ;
	*SIO0_XC = SIO_XC_DMODE  ;
	*SIO0_RC = SIO_RC_DMODE  ;

	*SIO0_RXTC	= SIO_TMCNTR_STOP;

	*SIO0_GC = SIO_GC_DMODE | SIO_GC_GO_MASK ;

 }

void init_sio_cmode()
 {
	*SIO0_GC = SIO_GC_CMODE  ;
	*SIO0_XC = SIO_XC_CMODE  ;
	*SIO0_RC = SIO_RC_CMODE  ;

	*SIO0_RXTC	= SIO_TMCNTR_STOP;
	*SIO0_RXTCNT	= 0;
	*SIO0_RXTPER	= SIO_XTMPERIOD_CMODE;
	*SIO0_RXTC	= SIO_TMCNTR_GO;

	*SIO0_GC = SIO_GC_CMODE | SIO_GC_GO_MASK ;
 }


void set_ac_config_data()
{
	scan_channels	= MCD->chan & 0xff;

	if(scan_channels>0xf)
	  {
	    fram_len	= 2;
	    num_fram	= 7;
	    chip0_en	= 0xffffffff;
	    chip1_en	= 0xffffffff;
	    chip2_en	= 0xffffffff;
	    chip3_en	= 0xffffffff;
	  }
	 else
	   if(scan_channels>0x3)
	    {
	      fram_len	= 1;
	      num_fram	= 3;
	      chip0_en	= 0xffffffff;
	      chip1_en	= 0xffffffff;
	      chip2_en	= 0x0;
	      chip3_en	= 0x0;
	    }
	    else
	     {
	      fram_len	= 0;
	      num_fram	= 1;
	      chip0_en	= 0xffffffff;
	      chip1_en	= 0x0;
	      chip2_en	= 0x0;
	      chip3_en	= 0x0;
	     }

	/* create CMODE words #0/#1 and DMODE word #1 */

	t_ac_cts_1_2.clb=0;     /* CLB bit =0 */
	t_ac_cts_1_2.olb=0;	/* select 1Vrms scale */
	t_ac_cts_1_2.mlb=0;	/* enable MIC +20dB amplifier */

        t_ac_cts_1_2.rsrv3=1;	/* reserved bits */
	t_ac_cts_1_2.rsrv2=0;
	t_ac_cts_1_2.rsrv1=0;

	t_ac_cts_1_2.df = 0;	/* 16-bit data */
	t_ac_cts_1_2.st	= 1;	/* stereo mode */
	t_ac_cts_1_2.hpf = 0;	/* disable high-pass filter */

				/* set CLK selection */
	t_ac_cts_1_2.dfr = MCD->fq & 0x7;


	t_acm_cts_3_4.mck = (((MCD->fq)>>3) & 0x1) +1;

	t_acm_cts_3_4.xen = 0;
	t_acm_cts_3_4.xclk= 1;
	t_acm_cts_3_4.bsel= fram_len;
	t_acm_cts_3_4.its = 1;
	t_acm_cts_3_4.dad = 0;	/* disable loopback and tests */
	t_acm_cts_3_4.enl = 0;
	t_acm_cts_3_4.test= 0;

	t_acs_cts_3_4.mck = 3;

	t_acs_cts_3_4.xen = 0;
	t_acs_cts_3_4.xclk= 0;
	t_acs_cts_3_4.bsel= fram_len;
	t_acs_cts_3_4.its = 1;
	t_acs_cts_3_4.dad = 0;	/* disable loopback and tests */
	t_acs_cts_3_4.enl = 0;
	t_acs_cts_3_4.test= 0;

	t_ac_cts_5_6.rsrv1 =0;
	t_ac_cts_5_6.rsrv2 =0;
	t_ac_cts_5_6.pio   =0;	/* PIO unused */

	t_ac_cts_7_8.rsrv1 =0;
	t_ac_cts_7_8.rsrv2 =0;
	t_ac_cts_7_8.ver   =0;  /* read-back the version value */


	t_ac_dts_5_6.lo    =MCD->lo;
	t_ac_dts_5_6.le    =MCD->le;
	t_ac_dts_5_6.he    =MCD->he;
	t_ac_dts_5_6.ro    =MCD->ro;
	t_ac_dts_5_6.se    =0;	/* speaker disable */
	t_ac_dts_5_6.adi   =0;


	t_ac_dts_7_8.lg    =MCD->lg;
	t_ac_dts_7_8.is    =MCD->is;
	t_ac_dts_7_8.pio   =0;
	t_ac_dts_7_8.ovr   =0;

	t_ac_dts_7_8.rg    =MCD->rg;
	t_ac_dts_7_8.ma    =MCD->ma;


	cmode_w00	=  (((*(unsigned int*) &t_ac_cts_1_2)<<16) & 0xffff0000)
			 | ((*(unsigned int*) &t_acm_cts_3_4) & 0xffff);
	cmode_w10	=  (((*(unsigned int*) &t_ac_cts_1_2)<<16) & 0xffff0000)
			 | ((*(unsigned int*) &t_acs_cts_3_4) & 0xffff);

	cmode_w1	=  (((*(unsigned int*) &t_ac_cts_5_6)<<16) & 0xffff0000)
			 | ((*(unsigned int*) &t_ac_cts_7_8) & 0xffff);

	dmode_w01	=  (((*(unsigned int*) &t_ac_dts_5_6)<<16) & 0xffff0000)
			 | ((*(unsigned int*) &t_ac_dts_7_8) & 0xffff);
	dmode_w11	= dmode_w01 ;



}


void rx_cmode_ccl()
{
	_x_wcount=0;		/* initialize R/X interrupt handlers parameters */
	_r_wcount=0;

	x_cmode=r_cmode=ML_ON;	/* restart receiver and transmitter */

	*SIO0_GC &= ~SIO_GC_GO_MASK ;
	*SIO0_GC |=  SIO_GC_GO_MASK ;


	SX0_IRQ_SET;		/* initialize TRANSMITTER interrupt	*/

       while(x_cmode!=0 || r_cmode!=0);		/* WAIT_CMODE_RX_OK;
				       		wait for CMODE_W0/W1 frames
				       		transferred and received */
	_delay_100us();		/* wait 100us to ensure that
				all data has been transferred */
}

int configure_ac()
	/* this fucnction generates config info for AC,
	   configures AC in CONTROL mode and then
	   goes to the DATA mode with DAC-scan and ADC-scan disabled.
	*/
  {  int ccl_count;

	INT_DISABLE;

        set_ac_config_data();	/* assemble AC configuration CONTROL/DATA bit slots */

	 CLEAR_CLB;


	SET_AC_CMODE;        /* set D/C control line to CONTROL mode */

	init_sio_cmode();	/* configure DSP SIO-0 port */

	SX0_IRQ_SET;		/* enable interrupts	*/
	SR0_IRQ_CLEAR;
	SX0_INT_ENABLE;
	SR0_INT_ENABLE;

	INT_ENABLE;	 /* enable CMODE transmission with CLB=0 */

	rx_cmode_ccl();	 /* perform two initial C-MODE transmit/receive cycles */

		rx_cmode_ccl();


		/* check for the returned CLB bit. If not set, then
		   repeat transmission max 100 times.
		   (This meets specs for CS4215 and AD1849).
		 */
        if(   ((cmode_rd_w00 & _cmode_CLB_mask & chip0_en)!=0)
	   || ((cmode_rd_w10 & _cmode_CLB_mask & chip1_en)!=0)
	   || ((cmode_rd_w20 & _cmode_CLB_mask & chip2_en)!=0)	
	   || ((cmode_rd_w30 & _cmode_CLB_mask & chip3_en)!=0)
	  )
	   {
	     for(ccl_count=0;ccl_count<100;ccl_count++)
		{ rx_cmode_ccl();
		}

		/* check for CLB again. If still set, then return
		   the HARDWARE ERROR.
		*/
        if(   ((cmode_rd_w00 & _cmode_CLB_mask & chip0_en)!=0)
	   || ((cmode_rd_w10 & _cmode_CLB_mask & chip1_en)!=0)
	   || ((cmode_rd_w20 & _cmode_CLB_mask & chip2_en)!=0)	
	   || ((cmode_rd_w30 & _cmode_CLB_mask & chip3_en)!=0)
	  )
			return(MERR_HW_ERR);

	   }

		/* check for read-back info from AC and compare to
		   the sent info. If does not match, then return
		   the HARDWARE	ERROR.
		*/
//        if((cmode_rd_w0 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
//			return(MERR_HW_ERR);
//
//        if((cmode_rd_w01 & _cmode_rd_w1_mask)!=(cmode_w1 & _cmode_rd_w1_mask))
//			return(MERR_HW_ERR);

	do
	{
   	   rx_cmode_ccl();
	}
	while(   ((cmode_rd_w00 & _cmode_rd_w0_mask)!=
		  (cmode_w00 & _cmode_rd_w0_mask) && chip0_en)
	      || ((cmode_rd_w01 & _cmode_rd_w1_mask)!=
	      	  (cmode_w1 & _cmode_rd_w1_mask) && chip0_en)
	      || ((cmode_rd_w10 & _cmode_rd_w0_mask)!=
	      	  (cmode_w10 & _cmode_rd_w0_mask) && chip1_en)
	      || ((cmode_rd_w11 & _cmode_rd_w1_mask)!=
	      	  (cmode_w1 & _cmode_rd_w1_mask) && chip1_en)
	      || ((cmode_rd_w20 & _cmode_rd_w0_mask)!=
	      	  (cmode_w10 & _cmode_rd_w0_mask) && chip2_en)
	      || ((cmode_rd_w21 & _cmode_rd_w1_mask)!=
	      	  (cmode_w1 & _cmode_rd_w1_mask) && chip2_en)
	      || ((cmode_rd_w30 & _cmode_rd_w0_mask)!=
	      	  (cmode_w10 & _cmode_rd_w0_mask) && chip3_en)
	      || ((cmode_rd_w31 & _cmode_rd_w1_mask)!=
	      	  (cmode_w1 & _cmode_rd_w1_mask) && chip3_en)
             );


		/* now transmit at least 2 frames with CLB=1
		   in order to lock AC and to prepare to go to the
		   DATA mode
		*/
	SET_CLB;

   	   rx_cmode_ccl();
   	   rx_cmode_ccl();
   	   rx_cmode_ccl();
   	   rx_cmode_ccl();


	INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;
	_x_wcount=0;		/* initialize R/X interrupt handlers parameters */
	_r_wcount=0;

	SET_AC_DMODE;    	/* set D/C control line to DATA mode */
	_delay_40ms();		// wait 40ms to ensure AUTOCAL off for AD1849
	init_sio_dmode();	/* configure DSP SIO-0 port */

	SX0_IRQ_SET;		/* enable interrupts	*/
	SR0_IRQ_CLEAR;
	SX0_INT_ENABLE;
	SR0_INT_ENABLE;

	INT_ENABLE;		/* enable D-MODE transmission */

	return(MERR_OK);

  }

	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  { int r;

	INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

       	r=configure_ac();

	if(r!=MERR_OK)	/* terminate if error is set */
	  {
	    INT_DISABLE;
	    return(r);
	  }

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) DAC;
        dac_get_ptr		= (int) DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;

	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_GO);	/* wait for MCD->status==GO */
        adc_scan_flag	= ML_ON;	/* enable saving ADC -data */

	while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */

	INT_DISABLE;
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */

	SX0_INT_DISABLE;
	SR0_INT_DISABLE;

	return(MERR_OK);
  }



	/* this procedure performs cyclic ADC->DAC scanning.
	*/
int	adc_to_dac()
  {int r;

	INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;


       	r=configure_ac();

	if(r!=MERR_OK)	/* terminate if error is set */
	  {
	    INT_DISABLE;
	    return(r);
	  }

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) ADC;
        dac_get_ptr		= (int) ADC;
	dac_data_count		= 0;
        dac_data_count_max	= 0x10; /* use 10-words cyclic buffer */


	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	dac_scan_flag		= ML_ON;
	adc_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_ABORT);

	INT_DISABLE;
        adc_scan_flag	= ML_OFF;
	dac_scan_flag	= ML_OFF;

	SX0_INT_DISABLE;
	SR0_INT_DISABLE;
	return(MERR_OK);
  }
