/***********************************************************************
 Demo for T54x with T/SDAS-S4215 AD/DA board test utility.
 DAC output and ADC Acquisition with pretriggering. No previous reset.
 Copyright (C) MicroLAB(tm) Systems Ltd, 1998-2002

*************************************************************************/



#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <mlu.h>
#include <ips_ut.h>

#include <t54x_h.c>
#include "t54_comm.i"

#define  SCAN_LEN_DEFAULT	(0x3c00L/8L)

long   scan_len,   // epoch length in samples
       kbd_cmnd,cmnd;       // amplitude
int    fq_code=8;
double fq=16000.0;       // Fq in Hz


long	i,chan=3;
long	real_chan=0xff;	// each channel (0..7) has its own enable bit

int	tmp,ex_status,adc_scan_flag=0;
char	s[80],ch;
int	dat_type=2; // def for INT type (number of bytes per word)

MLU_TBDD TB;  // time bell descriptor


#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");



struct MCD_DD  *MCD;     // MCD pointer

int    *D;               // master data buffer array

long   MADC_SADDR,
       MDAC_SADDR;

int    lo=0,		// S4215 specific parameters
       ro=0,
       ma=15L,
       lg=0,
       rg=0,
       le=1L,
       he=1L,
       is=0L;


struct	MCD_S4215_DD
	{
	  int	lo,	// left output attenuation   (0..63)
		ro,	// right output attenuation  (0..63)
		lg,	// left input gain           (0..15)
		rg,	// right input gain          (0..15)
		ma,	// monitor path attenuation  (0..15)
		le,	// line output enable        (0/1)
		he,	// headphone output enable   (0/1)
		is;	// input selector (0-line input, 1-mic input)
	  unsigned 	mcbsp_flag;	// ==1 for T54x with McBSP ports (T54xx: T5402/T5410/T5416)
							// ==0 for T54x with BSP ports (T542L/T548/T549)
	}
		*MCD_S4215 ;





inp_param()
{

  enter_sf:
      printf("sampling frequency (Fs) selection list: \n");
      printf("     0: 8k          8: 5.5125k\n");
      printf("     1: 16k         9: 11.025k\n");
      printf("     2: 27.42k     10: 18.9k\n");
      printf("     3: 32k        11: 22.05k\n");
      printf("     4: n/a        12: 37.8k\n");
      printf("     5: n/a        13: 44.1k\n");
      printf("     6: 48k        14: 33.075k\n");
      printf("     7: 9.6k       15: 6.615k\n");

      printf("enter Fs code #           : ");
      fq_code=(int)getlong(0L,15L,(long)fq_code,CR_ENABLE,ESC_MASK,&ex_status);

      if(ex_status==ESC_MASK)
       {aborted:
	  printf(" <aborted>\n");
	  return(1);
       }

      switch(fq_code)
       {
         case 0: fq=8000.0;
		 break;
         case 1: fq=16000.0;
		 break;
         case 2: fq=27420.0;
		 break;
         case 3: fq=32000.0;
		 break;

         case 4:
	 case 5:
                 printf("\nerror: illegal Fs \n\n");
		 beep_err();
 	         goto enter_sf;

         case 6: fq=48000.0;
		 break;
         case 7: fq=9600.0;
		 break;
         case 8: fq=5512.5;
		 break;
         case 9: fq=11025.0;
		 break;
         case 10: fq=18900.0;
		  break;
         case 11: fq=22050.0;
		  break;
         case 12: fq=37800.0;
		  break;
         case 13: fq=44100.0;
		  break;
         case 14: fq=33075.0;
		  break;
         case 15: fq=6615.0;
		  break;

	 default:
		  break;

       }

      printf("left  input gain (0..15)  : ");
      lg=(int)getlong(0L,15L,(long)lg,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;


      printf("right input gain (0..15)  : ");
      rg=(int)getlong(0L,15L,(long)rg,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("left  output atten (0..63): ");
      lo=(int)getlong(0L,63L,(long)lo,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("right output atten (0..63): ");
      ro=(int)getlong(0L,63L,(long)ro,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("monitor path atten (0..15): ");
      ma=(int)getlong(0L,15L,(long)ma,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("input select(0-line/1-mic): ");
      is=(int)getlong(0L,1L,(long)is,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("line output enable (0/1)  : ");
      le=(int)getlong(0L,1L,(long)le,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("headphones enable (0/1)   : ");
      he=(int)getlong(0L,1L,(long)he,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

	printf("\n");
      return(0);

}





/*****************************************************************************
// fill on-board SRAM with zero or sine waves with F0 and 2xF0 frequencies
*****************************************************************************/

#define FILL_SINE   1
#define FILL_ZERO   0

 int __fill_dac_sram(int fill_mode)
  {
    long count, start, q, n, n1, j, chan_count, w_len;



      CLEAR_CURRENT_LINE;
      printf("filling SRAM ...\r");

      for (chan_count=0; chan_count<8; chan_count++)
       {
	    start=MDAC_SADDR/2L;
	    j=start+(scan_len*chan_count);
	    count=scan_len;
	    n=0L;

	    do
	      {
                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++)
		  {
		    if(fill_mode==FILL_SINE)
		      {
			 switch(chan_count)
			  {
			   case 0:
			     *D++ = (long) ((double) (10000.0 *
			     sin(50.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 1:
			     *D++ = (long) ((double) (10000.0 *
			     sin(75.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 2:
			     *D++ = (long) ((double) (10000.0 *
			     sin(100.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    	    break;
			   case 3:
			     *D++ = (long) ((double) (10000.0 *
			     sin(150.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 4:
			     *D++ = (long) ((double) (10000.0 *
			     sin(200.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 5:
			     *D++ = (long) ((double) (10000.0 *
			     sin(300.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 6:
			     *D++ = (long) ((double) (10000.0 *
			     sin(400.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 7:
			     *D++ = (long) ((double) (10000.0 *
			     sin(600.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			  }
		      }
		     else
			  *D++ = 0L;
		  }

                if(T54X_sb_err_data())
	           {
		     CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
		n+=q;
	      }
	     while (count>0);
         }

     CLEAR_CURRENT_LINE;
     return(0);
  }



 int __read_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    int *LPTR;

            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {
                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);

                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *LPTR++= *D++);

                if(T54X_sb_err_data())
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


 int __write_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    int *LPTR;


            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *D++=*LPTR++ );

                if(T54X_sb_err_data())
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }






#define TMP_BUF_LEN	0x8000L


 int __move_adc_to_dac(long buf_len)
  {
    long count, j, q, jd;
    int *TMP;



      CLEAR_CURRENT_LINE;
      printf("moving SRAM data ...\r");


    if((TMP=(int *) halloc(TMP_BUF_LEN*2L,1))==NULL)

       { printf("error: out of memory\n\n");
	 return(1);
       }

                      /* copy ADC data to DAC buffer */

	    j=MADC_SADDR/2L;
	    jd=MDAC_SADDR/2L;
	    count=buf_len;

	    do
	      {
                if(count>=TMP_BUF_LEN) q=TMP_BUF_LEN;
		    else	q=count;
		if(   __read_sb_data(j,q,TMP)!=0
		   || __write_sb_data(jd,q,TMP)!=0
		  )
           	   {
                     hfree(TMP);return(1);
	           }
		count-=q;
		j+=q;
		jd+=q;
	      }
	     while (count>0);


    CLEAR_CURRENT_LINE;
    hfree(TMP);
    return(0);
  }



int do_cmd(int adc_scan_enable)
 {
  T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);


  MCD->cmnd		= cmnd;
  MCD->fq		= fq_code;
  MCD->epoch_len	= scan_len;
  MCD->status		= MSTAT_ATTN;
  MCD->chan		= real_chan;

  MCD_S4215->lo		= lo;
  MCD_S4215->ro		= ro;
  MCD_S4215->lg		= lg;
  MCD_S4215->rg		= rg;
  MCD_S4215->ma		= ma;
  MCD_S4215->le		= le;
  MCD_S4215->he		= he;
  MCD_S4215->is		= is;


  adc_scan_flag=0;

  T54X_SET_HM_RQ;  // set MASTER ATTN via interrupt

  printf(" T54x operation in progress. Press <SPACE> to terminate or <ESC> to abort...\r");
// printf("\n");

  do                      // wait for the ESC pressed or data ready
   {

/*
     printf("\n CHAN=%i CMND=%x Fq=%d LEN=%d STAT=%x err=%x SBUS_err=%d",
     MCD->chan,MCD->cmnd, MCD->fq, MCD->epoch_len,
     MCD->status, MCD->err, T54X_sb_err_data());
*/

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
 	      exe_aborted:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T54x reset>\n");
	      m_restart:
	         T54X_SET_MRESET;
		 T54X_CLR_SB_ERR;
	         return(1);

            case 0x20:  // issue ADC SCAN synch
                 if (adc_scan_enable)
		  {
                    if (!adc_scan_flag)
	    	      { CLEAR_CURRENT_LINE;
		        printf("\rADC scanning...\r");
		        adc_scan_flag=MLU_ON;
		        MCD->status=MSTAT_GO;
		      }
		  }
		  else
		  {
	            T54X_SET_MRESET;
		    T54X_CLR_SB_ERR;
		    T54X_SET_MGO;
		    return(0);
		  }
		 break;

            default:
		 break;

	  }
       }

   }
    while(MCD->status!=MSTAT_RDY);

   if (MCD->err!=MLU_OK)
     { CLEAR_CURRENT_LINE;
       printf("error (do_cmnd): %d\n", MCD->err);
       return (1);
     }
   return(0);
 }



int do_adc_to_dac_cmnd()
 {
  T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);

  MCD->cmnd		= cmnd;
  MCD->fq   		= fq_code;
  MCD->epoch_len	= 0x10;
  MCD->status		= MSTAT_ATTN;
  MCD->chan		= real_chan;


  MCD_S4215->lo		= lo;
  MCD_S4215->ro		= ro;
  MCD_S4215->lg		= lg;
  MCD_S4215->rg		= rg;
  MCD_S4215->ma		= ma;
  MCD_S4215->le		= le;
  MCD_S4215->he		= he;
  MCD_S4215->is		= is;

  adc_scan_flag=0;

  T54X_SET_HM_RQ;  // set MASTER ATTN via interrupt
  printf(" ADC->DAC is in progress. Press <SPACE> to terminate...\r");


  do                      // wait for the ESC pressed or data ready
   {

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
 	      exe_aborted:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T54x reset>\n");
	      m_restart:
	            T54X_SET_MRESET;
		    T54X_CLR_SB_ERR;
	         return(1);

            case 0x20:  // issue ADC SCAN synch
                 if (!adc_scan_flag)
		   { CLEAR_CURRENT_LINE;
		     printf("\rADC->DAC operation is aborted...\r");
		     adc_scan_flag=MLU_ON;
		     MCD->status=MSTAT_ABORT;
		   }
		 break;

            default:
		 break;

	  }
       }
   }
    while(MCD->status!=MSTAT_RDY);

   if (MCD->err!=MLU_OK)
     { CLEAR_CURRENT_LINE;
       printf("error: %ld\n", MCD->err);
       return (1);
     }
   return(0);
 }







main()

{  int r;
  char err_mess[100], dev_name[100];
  unsigned dev_id;


   printf("\nDemo Utility for T54x with T/SDAS-4215 Stereo Audio SIOX-module. v.1.1\n");
   printf("(C) MicroLAB Systems Ltd, 1998-2002\n\n");


   if(   (r=T54X_open(0x350))!=T54X_OK
      || (r=T54X_dev_id(&dev_id))!=T54X_OK
      || (r=T54X_dev_name(dev_name))!=T54X_OK
     )
     {  error:
          T54X_last_err_message(err_mess);
		  printf("error: %s\n", err_mess);
		  goto exit;
     }

   printf("DSP system:  %s, s/n: #%x\n\n", dev_name, dev_id & 0xff);


   T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);
   MCD_S4215 = (struct MCD_S4215_DD*) (& ((char*)&(MCD->chan))[2]);


   T54X_CLR_SB_ERR;

   T54X_SET_MRESET;       // init T54x and validate the response code

      MCD->m_id=0xffff;
      MCD->prg_id=0xffff;
      MCD->status=0xffff;
      MCD->err=0xffff;
      MCD->cmnd=0xffff;
      MCD->adc_data_saddr=0;
      MCD->dac_data_saddr=0;


		//--- pass info about the C54x/C54xx DSP type to the DSP
		//    resident software in order to provide correct arrangement
		//    of BSP/McBSP SIO port management
	MCD_S4215->mcbsp_flag	= (T542L_BOARD | T548_BOARD | T549_BOARD) ? 0:1;


   MLU_set_bell(1,&TB);  // set time control =1s
   T54X_SET_MGO;

   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
       if(MLU_check_bell(&TB)<0)
	   { printf("error: no reply from TORNADO-54x\n\n");
	     goto exit;
	   }
       if(T54X_sb_err_data())
	   { printf("error: SB error\n\n");
	     goto exit;
	   }
     }


   if(   MCD->m_id!=T54X_M_ID    // validate MCD returned
      || MCD->prg_id!=T54X_PRG_ID
      || MCD->status!=MSTAT_RDY
      || MCD->cmnd!=0
      || MCD->adc_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->adc_data_saddr>=(T54X_DRAM_EADDR-SCAN_LEN_DEFAULT)
      || MCD->dac_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->dac_data_saddr>=(T54X_DRAM_EADDR-SCAN_LEN_DEFAULT)
     )
       { printf("error: invalid T54x initialization data\n");
	 goto exit;
       }

   MADC_SADDR = ((long)MCD->adc_data_saddr)*2L;
   MDAC_SADDR = ((long)MCD->dac_data_saddr)*2L;
   scan_len=SCAN_LEN_DEFAULT;

   __fill_dac_sram(FILL_ZERO);


while(1)
 {
   printf("\n commands : \n");
   printf("      0 : Output F0/2xF0 sine waves via DAC-L/R\n");
   printf("      1 : Record L/R data in on-board SRAM\n");
   printf("      2 : Cyclic Playback L/R data from on-board SRAM\n");
   printf("      3 : Continuous pass-through (ADC->DAC)\n");
   printf("    Esc : exit to DOS\n\n");

   printf("enter command: ");

kbd_loop:
   switch(read_char())
     {
        case '0':
                CLEAR_CURRENT_LINE;
		printf("Output F0/2xF0 sine waves via DAC-L/R >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
                if(inp_param()!=0) goto exit;
		scan_len=SCAN_LEN_DEFAULT;

		if (    __fill_dac_sram(FILL_SINE)!=0
                     || do_cmd(MLU_OFF)!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
                   ) goto exit;
		break;


	case '1':

                CLEAR_CURRENT_LINE;
		printf("Record L/R data in on-board SRAM >>>>\n");
		cmnd = MCMND_DAS_DAC_SCAN;

                if(inp_param()!=0) goto exit;

		scan_len=SCAN_LEN_DEFAULT;

		if (    __fill_dac_sram(FILL_ZERO)!=0
                     || do_cmd(MLU_ON)!=0
		     || __move_adc_to_dac(2L*SCAN_LEN_DEFAULT)!=0
                   ) goto exit;
		break;




	case '2':
                CLEAR_CURRENT_LINE;
		printf("Play L/R data from on-board SRAM >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
                if(inp_param()!=0) goto exit;
		scan_len=SCAN_LEN_DEFAULT;

		if ( do_cmd(MLU_OFF)!=0
                   ) goto exit;
		break;



	case '3':

                CLEAR_CURRENT_LINE;
		printf("Continuous Pass-through L/R data (ADC->DAC) >>>>\n");
		cmnd = MCMND_ADC_TO_DAC;
                if(inp_param()!=0) goto exit;

		if (do_adc_to_dac_cmnd()!=0) goto exit;
                break;


        case KEY_ESC:
	  printf(" <aborted>\n");
	  goto exit;

	default:
		beep_err();
		goto kbd_loop;
     }

 }
exit:
  printf("\n");
}