/*****************************************************************
	T/SDAS-AU8/L demo/test utility for TORNADO-54x DSP Systems.
	(C) MicroLAB Systems Ltd, 1994-2002

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com

	Description:
	  1. This demo runs on TORNADO-54x DSP Systems for ISA-bus PC only and
	     does not run for stand-alone DSP applications.
	  2. This DSP code runs together with host PC application for DOS and
	     provides interactive selection of demo mode and programming of 
		 T/SDAS-AU8/L 8-channel audio DCM via simple host text-mode dialog.
	  3. Host DOS application performs initialization of this DSP code.
	  4. It is not possible to debug this DSP code using JTAG emulator
	     due to the host control of DSP RESET signal, which results in
		 JTAG debugger crash.
	

	Notes:
	  1. This demo runs with 2x/4x/8x AD/DA channels. The number of used
		 AD/DA chanels is defined via host DOS application.
	  2. BSP/McBSP0 SIO port is used for communication with
         T/SDAS-AU8/L DCM.
	  3. DSP software handles C54x DSPs with both BSP and McBSP
	     SIO ports and automatically configures SIO port.
	     Detection of C54xx DSPs (C5402/C5410/C5416) with McBSP ports
	     is performed by host software and information is passed
	     via the MCD->mcbsp_flag of MCD communication descriptor.

******************************************************************/


#include	<math.h>
#include	"c54x_def.h"
#include	"t54x_dsp.h"

// PC-to-DSP master communication descriptor (MCD)


struct	MCD_DD
	{ int	m_id,
			prg_id,
            status,
    		err,
			cmnd;

	  unsigned 	fq;
	  unsigned 	epoch_len,
               	adc_data_ptr,
               	dac_data_ptr,
				chan;

		// CS4215/AD1849 codec specific parameters for DMODE_B4..B7 of real-time DATA mode 
	int	lo,			// left output attenuation   (0..63)
		ro,			// right output attenuation  (0..63)
		lg,			// left input gain           (0..15)
		rg,			// right input gain          (0..15)
		ma,			// monitor path attenuation  (0..15)
		le,			// line output enable        (0/1) 
		he,			// headphone output enable   (0/1) 
		is,			// input selector (0-line input, 1-mic input) 
		mcbsp_flag;	// LSB==0 for C54x with BSP port (C542/C548/C549)
					// LSB==1 for C54xx with McBSP ports (C5402/C5410/C5416)
	} ;

// default address for MCD in D-SRAM is *(0xff00)
struct	MCD_DD *MCD	= (struct MCD_DD *)0xff00;


#define	T54X_ID				0x101
#define	PRG_ID				0x100

#define	MSTAT_RDY			0 	// Master status values
#define	MSTAT_BUSY			1
#define	MSTAT_ATTN			2
#define	MSTAT_GO			3
#define	MSTAT_ABORT			4

#define	MERR_OK				0 	// Master returned error codes
#define	MERR_INV_CMND		1 	//     ; invalid command 
#define	MERR_INV_ATTN		2
#define	MERR_HW_ERR			3

#define	MCMND_NOP			0
#define	MCMND_ABORT			1

#define	MCMND_DAS_SCAN		0x100
#define	MCMND_DAS_DAC_SCAN	0x101
#define	MCMND_DAC_SING_LOAD	0x102
#define	MCMND_ADC_TO_DAC	0x103

#define	ML_OFF				0
#define	ML_ON				1

#define	MDATA_BUF_LEN		(0x3c00/8) 	// maximum data array length


extern  unsigned int r_buf_start, x_buf_start;
int	*ADC,	*DAC;				// ptrs to the ADC/DAC data 

int	host_rq_flag=0;

int	mcbsp_flag=0;				// is set to '1' for T54xx with McBSP ports

#define	SIO_NOT_MCBSP	(mcbsp_flag==0)
#define	SIO_MCBSP		(mcbsp_flag==1)


// include T/SDAS-AU8/L application specific DSP code
#include	"t54_au8.i"


// host-to-DSP interrupt request handler
interrupt void c_xint3() 
{
	 host_rq_flag=1;
}




cpu_init()
 {
  *C54X_IMR=0;								// clear IMR since it is not cleared on reset
  *C54X_IFR=0;

  T54X_init_dsp();

  T54X_set_xiof1_dir(T54X_XIOF_OUTPUT); 	// use XIOF-1 to set D/C modes of AC
  T54X_set_xiof0_dir(T54X_XIOF_OUTPUT); 	// use XIOF-0 for performance tracking 

  T54X_set_pxsx_run(0x7);					// remove SIOX/PIOX reset
 }



int	cmnd_decoder()							// command decoder
 {
	if(MCD->status!=MSTAT_ATTN) return(MERR_INV_ATTN);

	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
		// common cmnds
	   case MCMND_NOP:						
	   case MCMND_ABORT:
		 return(MERR_OK);

       	// application specific commands
	   case MCMND_DAS_DAC_SCAN:
	   	 return( das_dac_scan() );

	   case MCMND_ADC_TO_DAC:
	   	 return( adc_to_dac() );

		// all unknown commands cause error
	   default:			
		 return( MERR_INV_CMND );
	 }

 }






void main()
{
	cpu_init();

	ADC	=   (int *) &r_buf_start ;
	DAC	=   (int *) &x_buf_start ;

	MCD->m_id		= T54X_ID;
	MCD->prg_id		= PRG_ID;
	MCD->cmnd		= 0;
	MCD->fq			= 0;
	MCD->epoch_len		= 0;
	MCD->err		= MERR_OK;
	MCD->adc_data_ptr	= (unsigned) &r_buf_start;
	MCD->dac_data_ptr	= (unsigned) &x_buf_start;
	MCD->status		= MSTAT_RDY;

	// copy McBSP port detector, which is set by host
	mcbsp_flag = (MCD->mcbsp_flag & 1);	


	// continuous loop
	while(1)			
	 {

		// poll for the HOST RQ
	   host_rq_flag=0;				

       C54X_XINT3_CLEAR;
	   C54X_XINT3_ENABLE;  				// enable IRQ-3 (HM_RQ)
       C54X_BSPR_INT_DISABLE;
       C54X_BSPX_INT_DISABLE;
	   C54X_INT_ENABLE;

			// wait until host IRQ occurs
       while(host_rq_flag==0) ;			

			// disable HOST RQ
	   C54X_XINT3_DISABLE;				

		// call exec & return error code and clear status flag 
	   MCD->err = cmnd_decoder();		
       MCD->status = MSTAT_RDY;			

	 }
}
