/*****************************************************************
	T/SDAS-ATEL2 demo/test utility.
	(C) MicroLAB Systems Ltd, 1994-1998

	notes:
	  1. SIO-0 DSP port is used for communication with
             T/SDAS-ATEL2 DCM.
	  2. DIN-0..3 data inputs are forwarded to DOUT-0..3 outputs
	     for transparent control.

******************************************************************/

// general includes
#include	<math.h>

// DSP master control descriptor (MCD)
struct	MCD_DD
{
	int			status,
				err,
				cmnd;

	unsigned 	fq;
	unsigned 	epoch_len,
				chan;		// AD/DA chanels involved

		// CS4215/AD1849 codec specific parameters for DMODE_B4..B7 of real-time DATA mode
	int			lo,			// left output attenuation   (0..63)
				ro,			// right output attenuation  (0..63)
				lg,			// left input gain           (0..15)
				rg,			// right input gain          (0..15)
				ma,			// monitor path attenuation  (0..15)
				le,			// line output enable        (0/1)
				he,			// headphone output enable   (0/1)
				is;			// input selector (0-line input, 1-mic input)
};
/*	default address for MCD in SRAM is *(0x40) */	
struct	MCD_DD *MCD	= (struct MCD_DD *)0x40;


#define	MSTAT_RDY				0 	// Master status values
#define	MSTAT_BUSY				1
#define	MSTAT_ATTN				2
#define	MSTAT_GO				3
#define	MSTAT_ABORT				4

#define	MERR_OK					0 	// Master returned error codes
#define	MERR_INV_CMND			1 	//     ; invalid command
#define	MERR_INV_ATTN			2
#define	MERR_HW_ERR				3

#define	MCMND_NOP				0
#define	MCMND_ABORT				1

#define	MCMND_DAC_SINE_OUTPUT	0x102
#define	MCMND_ADC_TO_DAC		0x103

#define	ML_OFF					0
#define	ML_ON					1

#define	MDATA_BUF_LEN			(0x7C00/4) 	// maximum data array length


int *ADC, *DAC;		/* ptrs to the ADC/DAC data */


//---------------------- include DEMO CNF file ---------------------------
#include	"saudio_c3x.cfg"		// include run-time compiler defs for demo mode and params

	// TORNADO-E3x specific definitions and include
#if(_E3X_SELECTED)
	#include	"c3x_def.h"
	#include	"te3x_dsp.h"	// include defs for E3x

	#define	_REMOVE_DCM_RESET	TE3X_set_pxsx_run(TE3X_SX_RUN)		// release SIOX-A reset
	#define	_SET_DCM_RESET		TE3X_set_pxsx_run(TE3X_SX_RESET)	// set SIOX-A reset
	#define	_DATA_BADDR			(TE3X_SRAM_SADDR + 0x100)			// base address for external audio AD/DA data
#endif


// include T/SDAS-AU8/L application specific DSP code
#include	"SAUDIO_C3X.I"


#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
// include DAC SINE fill-in function in case DEMO FUNCTION is DAC_SINE_OUTPUT

#define	TWO_PI			(3.1415926535 * 2.0)

// fill-in DAC buffers with sine data with different frequency for each channel
void fill_dac_sine()
{
	int *D;
	unsigned chan_count, n;
	double scan_len;

	scan_len = (double) MDATA_BUF_LEN;

	for (chan_count=0, D = DAC; chan_count < 2; chan_count++)
	{
		for (n = 0; n < MDATA_BUF_LEN; n++)
		{
			if (chan_count==0)
				*D++ = (int) ((double) (10000.0 * sin(500.0*TWO_PI*((double) n) / scan_len)));
			else
				*D++ = (int) ((double) (10000.0 * sin(1000.0*TWO_PI*((double) n) / scan_len)));
		}
	}
}
#endif


void cpu_init()
{
	*C3X_PBUS_GC=TE3X_PBUS_GC_DATA;
	// reset DCM in order to reset synchronization in case application has been
	// restarted several times without DSP hardware reset.
	_SET_DCM_RESET;
	_delay_40ms();
}


/* command decoder */
int cmnd_decoder()
{
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	{
		/* application specific commands */
		case MCMND_DAC_SINE_OUTPUT:
			return(das_dac_scan());

		case MCMND_ADC_TO_DAC:
			return(adc_to_dac());

		/* all unknown commands cause error */
		default:
			return(MERR_INV_CMND);
	}
}



void main()
{
	// init CPU
	cpu_init();

	// allocate ADC/DAC buffers
	// assign ADC/DAC pointers directly to SRAM upon the TORNADO board type (refer to defs above)
	ADC = (int *) _DATA_BADDR;
	DAC = (int *) (_DATA_BADDR + MDATA_BUF_LEN*8*2);

	// copy run-time compiler demo mode and params to the MCD descriptor
		// set number of AC AD/DA channels involved into the demo
	MCD->chan = _DAQ_CHANNELS_ ;

		// set demo function mode
	MCD->cmnd = _FUNC_ ;

		// set number of data samples per channel
	MCD->epoch_len = MDATA_BUF_LEN;

		// set AC configuration parameters, which will be programmed into AC on demo run
	MCD->fq = _AC_FS_;
	MCD->lo = _AC_LO_;
	MCD->ro = _AC_RO_;
	MCD->lg = _AC_LG_;
	MCD->rg = _AC_RG_;
	MCD->ma = _AC_MA_;
	MCD->le = _AC_LE_;
	MCD->he = _AC_HE_;
	MCD->is = _AC_IS_;


	// fill-in DAC sine array in case DAC SINE output function has been selected
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	// start command execution & return error code in case of any error
	MCD->err = cmnd_decoder();
	MCD->status = MSTAT_RDY;


error:
	// get here and loop forever on any error
	while(1);
}
