/********************************************************************************
	SIOX Audio AD/DA DCM (T/SDAS-ATEL2, T-SDAS-AU/8L) Stand-alone Demo for 
	TMS320C6xxx based DSP boards (TORNADO-6x/P6x/E6x and E2/6xxx DSP Systems/Controllers). v.2.0

	(C) MicroLAB Systems Ltd, 1994-2006

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com


	Description:
	------------
	  1. This demo runs on TMS320C6xxx based DSP boards (TORNADO-6x/P6x, TORNADO-E6x
	     and E2/6xxx DSP Controllers) within DSP environment only in stand-alone mode
		 and does not require host control.
	  2. This DSP demo has been designed to be user configured by means of
		 compiler run-time definitions inside SAUDIO_C6X.CFG file, that allows to
		 select different demo modes and parameters:
			- SIOX Audio AD/DA DCM selection (either of T/SDAS-ATEL2 and T/SDAS-AU/8L)
			- TORNADO platform selection (either TORNADO-6x, TORNADO-P6xxx, TORNADO-E6xxx
			  or TORNADO-E2/6xxx)
			- number of AD/DA analog signal acquisition channels involved (2x/4x/8x)
			- demo mode selection (either SINUS output via all DAC channels, or
			  ADC->DAC pass-through mode for ach AD/DA channel)
			- CS4215/AD1849 audio-codec (A/C) run-time parameters (sampling frequency,
			  input selection, input gain, output attenuation, etc)
	  3. MIRAGE-510DX/P510D or MIRAGE-Nxx JTAG emulator and TI C6000 CCS tools are required to
		 recompile this demo and to upload code to the DSP environment.
	  4. It is possible to debug DSP code using JTAG emulator.
	  5. TMS320C6xxx DSP on-chip McBSP0 serial port is used for communication with A/C chip(s) at
		 SIOX Audio AD/DA DCM (T/SDAS-ATEL2, T/SDAS-AU8/L).
	  6. SIOX XIO-1 I/O pin must be configured as OUTPUT and is used for
		 control of DATA/CONTROL mode input for SIOX Audio AD/DA DCM.
	  7. SIOX XIO-0 I/O pin must be configured as OUTPUT and is used as
		 optional performance tracking output.
	  8. The demo performance is as it is and results in at least 16kHz sampling frequency
     	 for all TORNADO-6x/P6x/E6x and E2/6xxx DSP systems/controllers running at 167MHz and higher
		 frequencies. The lower DSP frequencies may result in performance degradation.


	T/SDAS-AU8L daughter-card module installation:
	-----------------------------------------------
	  1. SIOX Audio AD/DA DCM (T/SDAS-ATEL2 or T/SDAS-AU8L) must be installed into SIOX-A site
  	  2. On-board J1 jumper of T/SDAS-AU/8L DCM must be configured for SIOX TM/XIO-1 line control.
		 On-board J1 jumper of T/SDAS-ATEL2 DCM must be configured for SIOX TM/XIO-1 line control.
  	  3. Configure TORNADO-6x/P6x/E6x and E2/6xxx on-board TM-0/1 OUTPUT ENABLE jumpers/switches to enable
		 SIOX TM/XIO-0/1 I/O pins as OUTPUTs (install jumpers and set switches to the 'ON' state).


	Loading Demo and Initialization:
	---------------------------------
		All demos can be loaded into DSP environment via MIRAGE JTAG emulator
		(or compatible). Note, that the OVLY bit must be set to '1' prior loading
		the program in order to ensure that program area is loaded to on-chip memory
		for sufficient system performance.


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


	file: main .C file

*********************************************************************************/


// general includes
#include	"math.h"
#include	"stdlib.h"

// DSP master control descriptor (MCD)

struct	MCD_DD
{
	int			status,
    			err,
				cmnd;

	unsigned 	fq;
	unsigned 	epoch_len,
				chan;		// AD/DA chanels involved

		// CS4215/AD1849 codec specific parameters for DMODE_B4..B7 of real-time DATA mode
	int			lo,			// left output attenuation   (0..63)
				ro,			// right output attenuation  (0..63)
				lg,			// left input gain           (0..15)
				rg,			// right input gain          (0..15)
				ma,			// monitor path attenuation  (0..15)
				le,			// line output enable        (0/1)
				he,			// headphone output enable   (0/1)
				is;			// input selector (0-line input, 1-mic input)
	} ;

// default address for MCD in XRAM
struct	MCD_DD *MCD	= (struct MCD_DD *)0x200;

#define	MSTAT_RDY				0 	// Master status values
#define	MSTAT_BUSY				1
#define	MSTAT_ATTN				2
#define	MSTAT_GO				3
#define	MSTAT_ABORT				4

#define	MERR_OK					0 	// Master returned error codes
#define	MERR_INV_CMND			1 	//     ; invalid command
#define	MERR_INV_ATTN			2
#define	MERR_HW_ERR				3

#define	MCMND_NOP				0
#define	MCMND_ABORT				1

#define	MCMND_DAC_SINE_OUTPUT	0x102
#define	MCMND_ADC_TO_DAC		0x103

#define	ML_OFF					0
#define	ML_ON					1

#define	MDATA_BUF_LEN			(32000/(16*2)) 	// maximum data array length


int	short	*ADC=NULL,
			*DAC=NULL;				// ptrs to the ADC/DAC data



//---------------------- include DEMO CNF file ---------------------------
#include	"saudio_c6x.cfg"		// include run-time compiler defs for demo mode and params



	// TORNADO-E6x specific definitions and include
	// (C6x based demos shall be compiled with the AU8L_DEMO_C6x.LNK file copied to AU8L_C6X.LNK file)
#if(_E6X_SELECTED)
	#include	"c6x_def.h"
	#include	"te6x_dsp.h"	// include defs for E6x

	#define	_REMOVE_DCM_RESET	TE6X_set_pxsx_run(0x7)	// release SIOX-A reset
	#define	_SET_DCM_RESET		TE6X_set_pxsx_run(0)	// set SIOX-A reset

	#define	_DATA_BADDR			(TE6X_SBSRAM_SADDR+0x1000)		// base address for external audio AD/DA data
#endif

	// TORNADO-E64xx specific definitions and include
	// (C64xx based demos shall be compiled with the AU8L_DEMO_C64xx.LNK file copied to AU8L_C6X.LNK file)
#if(_E64XX_SELECTED)
	#define		__TE6X_SELECT_E64XX__	1
	#include	"c6x_def.h"
	#include	"te6x_dsp.h"	// include defs for E6x

	#define	_REMOVE_DCM_RESET	TE6X_set_pxsx_run(0x7)	// release SIOX-A reset
	#define	_SET_DCM_RESET		TE6X_set_pxsx_run(0)	// set SIOX-A reset

	#define	_DATA_BADDR			(C64XX_ONCHIP_RAM_SADDR+((3*C64XX_ONCHIP_RAM_LEN)/2))		// base address for external audio AD/DA data
#endif

	// TORNADO-E2/6713 specific definitions and include
	// (C6x based demos shall be compiled with the AU8L_DEMO_C6x.LNK file copied to AU8L_C6713.LNK file)
#if(_T2E6713_SELECTED)
	#define __T2E6XXX_SELECT_E6713__				1
	#include	"t2e6xxx_dsp.h"	// include defs for E2/6xxx

	#define	_REMOVE_DCM_RESET	T2E6XXX_SET_SIOX_DCM_RESET_OFF	// release SIOX-A reset
	#define	_SET_DCM_RESET		T2E6XXX_SET_SIOX_DCM_RESET_ON	// set SIOX-A reset

	#define	_DATA_BADDR			T2E6XXX_SBSRAM_AREA_SADDR		// base address for external audio AD/DA data
#endif

	// TORNADO-6x specific definitions and include
	// (C6x based demos shall be compiled with the AU8L_DEMO_C6x.LNK file copied to AU8L_C6X.LNK file)
#if(_T6X_SELECTED)
	#include	"c6x_def.h"
	#include	"t6x_dsp.h"		// include defs for T6x

	#define	_REMOVE_DCM_RESET	T6X_set_pxsx_run(0x7)	// release SIOX-A reset
	#define	_SET_DCM_RESET		T6X_set_pxsx_run(0)		// set SIOX-A reset

	#define	_DATA_BADDR			(T6X_SBSRAM_SADDR+0x1000)		// base address for external audio AD/DA data
#endif

	// TORNADO-P6x specific definitions and include
	// (C6x based demos shall be compiled with the AU8L_DEMO_C6x.LNK file copied to AU8L_C6X.LNK file)
#if(_TP6X_SELECTED)
	#include	"c6x_def.h"
	#include	"tp6x_dsp.h"	// include defs for TP6x

	#define	_REMOVE_DCM_RESET	TP6X_set_pxsx_run(0x7)	// release SIOX-A reset
	#define	_SET_DCM_RESET		TP6X_set_pxsx_run(0)	// set SIOX-A reset

	#define	_DATA_BADDR			TP6X_SBSRAM_SADDR		// base address for external audio AD/DA data
#endif


	// TORNADO-P64xx specific definitions and include
	// (C64xx based demos shall be compiled with the AU8L_DEMO_C64xx.LNK file copied to AU8L_C6X.LNK file)
#if(_TP64XX_SELECTED)
	#define		__TP6X_SELECT_TP64XX__	1
	#include	"c6x_def.h"
	#include	"tp6x_dsp.h"	// include defs for TP6x

	#define	_REMOVE_DCM_RESET	TP6X_set_pxsx_run(0x7)	// release SIOX-A reset
	#define	_SET_DCM_RESET		TP6X_set_pxsx_run(0)	// set SIOX-A reset

	#define	_DATA_BADDR			(C64XX_ONCHIP_RAM_SADDR+((3*C64XX_ONCHIP_RAM_LEN)/2))		// base address for external audio AD/DA data
#endif




	// SIOX TM/XIO-0/1 I/O pins control
	// (TORNADO on-board TM-0/1 OUTPUT ENABLE jumpers/switches shall be set to the 'ON' state)
#define TM_OUT_0  0
#define TM_OUT_1  4

#define	_SET_XIOF1_OFF_		*C6X_TM1_CONTROL=TM_OUT_0
#define	_SET_XIOF1_ON_		*C6X_TM1_CONTROL=TM_OUT_1
#define	_SET_XIOF0_OFF_		*C6X_TM0_CONTROL=TM_OUT_0
#define	_SET_XIOF0_ON_		*C6X_TM0_CONTROL=TM_OUT_1


	// McBSP R/X transmitter interrupts mapping
	// change these defs and macros if different interrupts is used
#define	_MCBSP_RINT				5
#define	_MCBSP_XINT				4

#define	_MCBSP_RINT_CLEAR_		C6X_INT_CLEAR(_MCBSP_RINT)
#define	_MCBSP_XINT_CLEAR_		C6X_INT_CLEAR(_MCBSP_XINT)

#define	_MCBSP_RINT_SET_		C6X_INT_SET(_MCBSP_RINT)
#define	_MCBSP_XINT_SET_		C6X_INT_SET(_MCBSP_XINT)

#define	_MCBSP_RINT_ENABLE_		C6X_INT_ENABLE(_MCBSP_RINT)
#define	_MCBSP_XINT_ENABLE_		C6X_INT_ENABLE(_MCBSP_XINT)

#define	_MCBSP_RINT_DISABLE_	C6X_INT_DISABLE(_MCBSP_RINT)
#define	_MCBSP_XINT_DISABLE_	C6X_INT_DISABLE(_MCBSP_XINT)

//-----------------------------------------------------------------------

// include T/SDAS-AU8/L application specific DSP code
#include	"saudio_c6x.i"


#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
// include DAC SINE fill-in function in case DEMO FUNCTION is DAC_SINE_OUTPUT

#define	TWO_PI			(3.1415926535 * 2.0)

void fill_dac_sine()
// fill-in DAC buffers with sine data with different frequency for each channel
{
	int short *D;
    unsigned  chan_count,n;
	float scan_len;

   	scan_len=(float)MDATA_BUF_LEN;

    for (chan_count=0,D=DAC; chan_count<8; chan_count++)
    {
    	for(n=0;n<MDATA_BUF_LEN;n++)
		{
			switch(chan_count)
			{
				case 0:
			    	*D++ = (int short) ((float) (10000.0*sin(20.0*TWO_PI*((float)n)/scan_len)));
			      	break;

			   	case 1:
			     	*D++ = (int short) ((float) (10000.0*sin(40.0*TWO_PI*((float)n)/scan_len)));
			     	break;

			   	case 2:
			     	*D++ = (int short) ((float) (10000.0*sin(60.0*TWO_PI*((float)n)/scan_len)));
		    	 	break;

			   	case 3:
			     	*D++ = (int short) ((float) (10000.0*sin(80.0*TWO_PI*((float)n)/scan_len)));
			     	break;

			   	case 4:
			     	*D++ = (int short) ((float) (10000.0*sin(100.0*TWO_PI*((float)n)/scan_len)));
			     	break;

			   	case 5:
			     	*D++ = (int short) ((float) (10000.0*sin(150.0*TWO_PI*((float)n)/scan_len)));
			     	break;

			   	case 6:
			     	*D++ = (int short) ((float) (10000.0*sin(200.0*TWO_PI*((float)n)/scan_len)));
			     	break;

			   	case 7:
			     	*D++ = (int short) ((float) (10000.0*sin(300.0*TWO_PI*((float)n)/scan_len)));
			     	break;
			}
		}
    }
}
#endif




int cpu_init()
// performs DSP initialization
{
	// clear & disable all IRQs
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	// perform INT mapping for McBSP X/R interrupts
	if(   C6X_INTMUX_SET(_MCBSP_XINT,C6X_INTSEL_MCBSP0_XINT)!=MERR_OK
	   || C6X_INTMUX_SET(_MCBSP_RINT,C6X_INTSEL_MCBSP0_RINT)!=MERR_OK
	  )
		return(-1);


	// perform TORNADO-E6x/E64xx DSP System initialization, set DSP type
#if(_E6X_SELECTED || _E64XX_SELECTED)
		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);
#endif

	// perform TORNADO-E2/6713 DSP System initialization
#if(_T2E6713_SELECTED)
	// initialize DSP
	T2E6XXX_init_dsp(T2E6713_EMIF_CLK_SEL_PLL_CLK);
	// enable Timers outputs through Serial ports configuration function
	T2E6XXX_set_dsp_ports_cnf(T2E6713_DSP_PORTS_CNF_MCBSP0_TOUT0_TOUT1);
	// enable SX_TM/XIO outputs as Timers outputs
	T2E6XXX_SET_SIOX_XIO0_CNF_TOUT0;
	T2E6XXX_SET_SIOX_XIO1_CNF_TOUT1;
#endif


	// perform TORNADO-6x DSP System initialization, set DSP type
#if(_T6X_SELECTED)
		// init DSP EMIF
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif


	// perform TORNADO-P6x DSP System initialization, set DSP type
#if(_TP6X_SELECTED)
		// init DSP EMIF
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif

	// perform TORNADO-P64xx DSP System initialization, set DSP type
#if(_TP64XX_SELECTED)
		// init DSP EMIF
	TP6X_init_dsp_emif();
#endif


	// reset DCM in order to reset synchronization in case application has been
	// restarte several times without DSP hardware reset.
	_SET_DCM_RESET;
	_wait_for_dcm_reset();

	// clear codec C/D control to the 'C' state and trace flag to the 'OFF' state
	_SET_XIOF0_OFF_;
	_SET_XIOF1_OFF_;

	return(0);
}




int cmnd_decoder()
// command decoder
{
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	{
       	// application specific commands
		case MCMND_DAC_SINE_OUTPUT:
	   	 	return( das_dac_scan() );

	   	case MCMND_ADC_TO_DAC:
	   	 	return( adc_to_dac() );

		// all unknown commands cause error
	   	default:
		 	return( MERR_INV_CMND );
	}
}






void main()
{
	// first, disable IRQs if this is the program restart under emulator
	// without DSP reset
	C6X_GINT_DISABLE;

	// init CPU
	if (cpu_init() != MERR_OK)
		goto error;


	// allocate ADC/DAC buffers
		//	Allocating ADC/DAC pointers from the heap will result to allocation to slow asynchronous DPRAM memory
		//  for TORNADO-P6x boards unless the .LNK linker command file is not changed for TP6x boards only, that
		//	is not convenient for the demo. User can uncomment these two lines below should ADC/DAC buffers allocation
		//	is considered from heap, or can use direct assignment using next operators.
//		if(   	(ADC=(int short *) malloc(MDATA_BUF_LEN*8*2))==NULL
//	 		|| 	(DAC=(int short *) malloc(MDATA_BUF_LEN*8*2))==NULL
//	  	  )
//			goto error;

		// assign ADC/DAC pointers directly to SBSRAM upon the TORNADO board type (refer to defs above)
		ADC = (int short *) _DATA_BADDR;
		DAC = (int short *) (_DATA_BADDR + MDATA_BUF_LEN*8*2);


	// copy run-time compiler demo mode and params to the MCD descriptor
		// set number of AC AD/DA channels involved into the demo
	MCD->chan = _DAQ_CHANNELS_ ;

		// set demo function mode
	MCD->cmnd = _FUNC_ ;

		// set number of data samples per channel
	MCD->epoch_len = MDATA_BUF_LEN;

		// set AC configuration parameters, which will be programmed into AC on demo run
	MCD->fq = _AC_FS_;
	MCD->lo = _AC_LO_;
	MCD->ro = _AC_RO_;
	MCD->lg = _AC_LG_;
	MCD->rg = _AC_RG_;
	MCD->ma = _AC_MA_;
	MCD->le = _AC_LE_;
	MCD->he = _AC_HE_;
	MCD->is = _AC_IS_;


	// fill-in DAC sine array in case DAC SINE output function has been selected
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	// start command execution & return error code in case of any error
   	MCD->err = cmnd_decoder();
	MCD->status = MSTAT_RDY;


error:
	// get here and loop forever on any error
	while(1);
}
