/****************************************************************************
	Application specific execution module.
	DCM: 				SIOX Audio AD/DA DCM (T/SDAS-ATEL2, T/SDAS-AU/8L)
	modes: 				D/A SINE OUTPUT, A/D->D/A pass-through,
						all modes utilize 2-ch D/A and 2ch A/D
	host DSP platform: 	TMS320C6xxx (TORNADO-6x/P6x/E6x and E2/6xxx)
*****************************************************************************/

/*
   Control/Data time slots for CS4215 Audio-codec chip.
   Organized as 16-bit words for demo portability between 32-bit and 16-bit DSPs.
   The lower significant time slots should be allocated into MSB of 16-bit word.
   User may want to redesign this demo for 32-bit DSP in order to get better performance.

   Declaration of CS4215/AD1849KP command data structures.
*/


typedef struct     	/* control time slots 1 & 2 */
{
	unsigned int df     :2; /* CTS2:  DF-0/1 */
	unsigned int st     :1;
	unsigned int dfr    :3; /*        DFR-0..2*/
	unsigned int rsrv1  :1;
	unsigned int hpf    :1;

	unsigned int rsrv2  :2; /* CTS1: reserved bits */
	unsigned int clb    :1;
	unsigned int olb    :1;
	unsigned int mlb    :1;
	unsigned int rsrv3  :3; /* should be ==001 always */
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_CTS_1_2;


typedef struct     	/* control time slots 3 & 4 */
{
	unsigned int dad    :1; /* CTS4*/
	unsigned int enl    :1;
	unsigned int test   :6;

	unsigned int xen    :1; /* CTS3 */
	unsigned int xclk   :1;
	unsigned int bsel   :2;
	unsigned int mck    :3;
	unsigned int its    :1;
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_CTS_3_4;


typedef struct     	/* control time slots 5 & 6 */
{
	unsigned int rsrv1  :8; /* CTS6 is reserved*/

	unsigned int rsrv2  :6; /* CTS5*/
	unsigned int pio    :2;
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_CTS_5_6;


typedef struct     	/* control time slots 7 & 8 */
{
	unsigned int rsrv1  :8; /* CTS8 is reserved*/

	unsigned int ver    :4; /* CTS7*/
	unsigned int rsrv2  :4;
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_CTS_7_8;


typedef struct     	/* data time slots 1 & 2 */
{
	int data_l  :16; /* DTS-1/2, left ADC/DAC data */
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_DTS_1_2;


typedef struct     	/* data time slots 3 & 4 */
{
	int data_r  :16; /* DTS-3/4, right ADC/DAC data */
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_DTS_3_4;


typedef struct     	/* data time slots 5 & 6 */
{
	unsigned int ro    :6; /* DTS-6, right output attenuation */
	unsigned int se    :1;
	unsigned int adi   :1;

	unsigned int lo    :6; /* DTS-5 */
	unsigned int le    :1;
	unsigned int he    :1;
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_DTS_5_6;


typedef struct     	/* data time slots 7 & 8 */
{
	unsigned int rg    :4; /* DTS-8 */
	unsigned int ma    :4;

	unsigned int lg    :4; /* DTS-7 */
	unsigned int is    :1;
	unsigned int ovr   :1;
	unsigned int pio   :2;
	unsigned int _x16  :16; // 32-bit word alignement supplement
} _AC_DTS_7_8;


// 16-bit CONTROL time slots
_AC_CTS_1_2	t_ac_cts_1_2;
_AC_CTS_3_4	t_acm_cts_3_4;
_AC_CTS_3_4	t_acs_cts_3_4;
_AC_CTS_5_6	t_ac_cts_5_6;
_AC_CTS_7_8	t_ac_cts_7_8;

unsigned short
		  	cmode_w0,			// transmitted 16-bit control time slots
            cmode_w1_m,
            cmode_w1_s,
            cmode_w2,
            cmode_w3,
			cmode_rd_w00,		// received 16-bit control time slots
			cmode_rd_w01,
			cmode_rd_w02,
			cmode_rd_w03,
			cmode_rd_w10,
			cmode_rd_w11,
			cmode_rd_w12,
			cmode_rd_w13,
			cmode_rd_w20,
			cmode_rd_w21,
			cmode_rd_w22,
			cmode_rd_w23,
			cmode_rd_w30,
			cmode_rd_w31,
			cmode_rd_w32,
			cmode_rd_w33;


// 16-bit data time slots
_AC_DTS_5_6	t_ac_dts_5_6;
_AC_DTS_7_8	t_ac_dts_7_8;

unsigned short
		  	dmode_w02,			// transmitted 16-bit data time slots
            dmode_w03,
			dmode_w12,
			dmode_w13,
			dmode_w22,
			dmode_w23,
			dmode_w32,
			dmode_w33,
            dmode_rd_w02,		// received 16-bit data time slots
			dmode_rd_w03,
            dmode_rd_w12,
			dmode_rd_w13,
            dmode_rd_w22,
			dmode_rd_w23,
            dmode_rd_w32,
			dmode_rd_w33,
			rw2,rw3,ww2,ww3;	// used for T/SDAS-ATEL2 to copy DIN-0..3 input bits to DOUT-0..3 output bits


// AND masks for verification of read-back data in CONTROL mode
#define	_cmode_rd_w0_mask	0x0c3f 	// AND mask for CMODE_W0 to mask reserved bits
#define	_cmode_rd_w1_mask	0xbf03 	// AND mask for CMODE_W1 to mask reserved bits
#define _cmode_rd_w2_mask	0x0000 	// AND mask for CMODE_W2 to mask reserved bits
#define _cmode_rd_w3_mask	0x0000 	// AND mask for CMODE_W3 to mask reserved bits

#define	_cmode_CLB_mask		0x0400 	// CLB bit mask applies to CMODE_W0

// AND masks to extract DIN-0..3 digital input data bits and to
// set DOUT-0..3 digital output bits for T/SDAS-ATEL2 DCM
#define _atel2_dio01_mask	0xc000 	// AND mask dmode_rd_w02 to extract X_IN
#define _atel2_dio23_mask	0xc000 	// AND mask dmode_rd_w03 to extract X_IN


// MACROS for setting CS4215/AD1849KP CONTROL/DATA mode via SIOX TM/XIO line
// Default is C/D control via SIOX TM/XIO-1 pin. Modify these macros to perform control via
// different SIOX TM/XIO pin.
#define	SET_AC_CMODE		{_SET_XIOF1_OFF_;rx_mode=RX_CMODE;}
#define	SET_AC_DMODE		{_SET_XIOF1_ON_;rx_mode=RX_DMODE;}

// McBSP port access macros
// Default is McBSP-0 serial port used for communication with AC chip.
// Modify these macros in case McBSP port different from McBSP-0 is used
#define	_SIO_XDATA			(*C6X_MCBSP0_DXR)
#define	_SIO_RDATA			(*C6X_MCBSP0_DRR)

#define	_SIO_SPCR			(*C6X_MCBSP0_SPCR)
#define	_SIO_PCR			(*C6X_MCBSP0_PCR)

#define	_SIO_XCR			(*C6X_MCBSP0_XCR)
#define	_SIO_RCR			(*C6X_MCBSP0_RCR)

#define	_SIO_SRGR			(*C6X_MCBSP0_SRGR)

#define	_SIO_RESET			_SIO_SPCR=0



/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

    The tracking signal is SIOX TM/XIO-0 signal from the SIOX site connector.

	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to the pin TM/XIO-0 of on-board SIOX connector.
	(refer to TORNADO-6x/P6x/E6x documentation for SIOX connector details).
*/

#define _TIME_TRACKING_SET_0	_SET_XIOF0_OFF_
#define _TIME_TRACKING_SET_1	_SET_XIOF0_ON_



// X/R interrupt handler parameters
unsigned 	dac_scan_flag			= ML_OFF,
			dac_cyclic_scan_flag	= ML_ON,
			dac_data_count,
        	dac_data_count_max,
			x_cmode					= 0;


unsigned	adc_scan_flag			= ML_OFF,
			adc_cyclic_scan_flag	= ML_OFF,
			adc_data_count,
			adc_data_count_max,
			r_cmode					= 0;

int short	*dac_data_start_ptr	= NULL,	// run-time data pointers
        	*dac_get_ptr		= NULL,
			*adc_save_ptr		= NULL,
			*adc_data_start_ptr	= NULL;


int			rx_mode	= 0;    		// C-MODE or D-MODE
int			scan_channels=0;		// number of channels for SCAN
int			frame_len=0;			// define .bsel feild vs. channels for SCAN
int			num_frame=3;			// (number of 16-bit word per frame)-1 (used by T/R ISRs)


#define	RX_CMODE			0
#define RX_DMODE			1

#define CHAN_0				1		// scanned channel IDs: left-channel AC #0
#define CHAN_1				2		// left  channel AC #0
#define CHAN_2				4		// right channel AC #1
#define CHAN_3				8		// left  channel AC #1
#define CHAN_4				0x10	// right channel AC #2
#define CHAN_5				0x20	// left  channel AC #2
#define CHAN_6				0x40	// right channel AC #3
#define CHAN_7				0x80	// left  channel AC #3

	// defs for different AD/DA channels configurations
#define DAQ_CH8_MODE   		(num_frame==15)
#define DAQ_CH4_MODE   		(num_frame==7)
#define DAQ_CH2_MODE	   	(num_frame==3)
#define DAQ_CH4_OR_CH8_MODE	(num_frame>=7)


#define WAIT_CMODE_RX_OK	while(x_cmode!=0 || r_cmode!=0);

	// macros for CLB control
#define CLEAR_CLB			(cmode_w0 &=~_cmode_CLB_mask);
#define SET_CLB				(cmode_w0 |= _cmode_CLB_mask);




//----------------------- McBSP interrupts ------------------------------

//~~~~~~~~ McBSP transmitter interrupt
int 			_x_wcount=0;	// transmitter word counter
unsigned short	dxr;			// data to be transmitted


interrupt void sio_xint()
{
//   _TIME_TRACKING_SET_1;	// set PERFORMACE TRACKING FLAG

	if(rx_mode==RX_CMODE)	// CONTROL MODE
   	{
  		switch(_x_wcount)     // decode word # and transmit the word
   		{
     		case 0:
     		case 4:
     		case 8:
     		case 12:
	  			dxr=cmode_w0;
	  			break;

       		case 1:
	  			dxr=cmode_w1_m;
	  			break;

       		case 2:
       		case 6:
       		case 10:
       		case 14:
	  			dxr=cmode_w2;
	  			break;

       		case 3:
       		case 7:
       		case 11:
       		case 15:
	  			dxr=cmode_w3;
	  			break;

       		case 5:
       		case 9:
       		case 13:
	  			dxr=cmode_w1_s;
	  			break;
		}

		// transmit data
	   	_SIO_XDATA = (unsigned) dxr;


		// check for cmode transmission is complete.
		// If YES, then clear X_CMODE flag for background control.
	   	if(_x_wcount == num_frame)
	   		x_cmode=ML_OFF;
	}
  	else
	{
		// DATA mode
		// decode word #
	  	switch(_x_wcount)
	   	{
	    	case 0:				// transmit DAC channel #0
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0) ? (*dac_get_ptr):0;
		  		break;

         	case 1:				// transmit DAC channel #1
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0) ? (dac_get_ptr[dac_data_count_max]):0;
		  		break;

         	case 2:				// transmit DMODE_W02
		  		dxr=dmode_w02;
		  		break;

         	case 3:				// transmit DMODE_W03
		  		dxr=dmode_w03;
		  		break;

	     	case 4:				// transmit DAC channel #2
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_2)!=0) ? (dac_get_ptr[2*dac_data_count_max]):0;
		  		break;

         	case 5:				// transmit DAC channel #3
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_3)!=0) ? (dac_get_ptr[3*dac_data_count_max]):0;
		  		break;

         	case 6:				// transmit DMODE_W12
		  		dxr=dmode_w12;
		  		break;

         	case 7:				// transmit DMODE_W13
		  		dxr=dmode_w13;
		  		break;

	     	case 8:				// transmit DAC channel #4
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_4)!=0) ? (dac_get_ptr[4*dac_data_count_max]):0;
		  		break;

         	case 9:				// transmit DAC channel #5
		  		dxr= ((dac_scan_flag==ML_ON) && (scan_channels & CHAN_5)!=0) ? (dac_get_ptr[5*dac_data_count_max]):0;
		  		break;

         	case 10:				// transmit DMODE_W22
		  		dxr=dmode_w22;
		  		break;

         	case 11:				// transmit DMODE_W23
		  		dxr=dmode_w23;
		  		break;

	     	case 12:				// transmit DAC channel #6
		  		dxr=((dac_scan_flag==ML_ON) && (scan_channels & CHAN_6)!=0) ? (dac_get_ptr[6*dac_data_count_max]):0;
		  		break;

         	case 13:				// transmit DAC channel #7
		  		dxr=((dac_scan_flag==ML_ON) && (scan_channels & CHAN_7)!=0) ? (dac_get_ptr[7*dac_data_count_max]):0;
		  		break;

         	case 14:				// transmit DMODE_W32
		  		dxr=dmode_w32;
		  		break;

         	case 15:				// transmit DMODE_W33
		  		dxr=dmode_w33;
		  		break;
		}

		// transmit data
	   	_SIO_XDATA = (unsigned) dxr;


		// check for update pointers and counters during last Tx-cycle in data mode
	   	if(num_frame==_x_wcount)
	    {
			if(dac_scan_flag==ML_ON)
            {
				dac_get_ptr++;

                if(++dac_data_count >=dac_data_count_max)
       	        {
					if(dac_cyclic_scan_flag==ML_ON)
				    {
						dac_data_count=0;
                        dac_get_ptr=dac_data_start_ptr;
		      		}
				    else
				    	dac_scan_flag=ML_OFF;
		     	}
			}
		}
	}


	// increment transmitter word counter
    _x_wcount++;
	_x_wcount %=(num_frame+1);

//   _TIME_TRACKING_SET_0;        // clear PERFORMANCE TRACKING FLAG

}


//~~~~~~~~~~~ receiver interrupt
int 		_r_wcount=0;				// receiver word counter
int short 	_r_data;

interrupt void sio_rint()
{
   _TIME_TRACKING_SET_1;		// set PERFORMACE TRACKING FLAG

	// read receiver data
	// extend 16-bit signed data to 32-bit signed value
   _r_data = (_SIO_RDATA & 0xffff);


   if(rx_mode==RX_CMODE)
   {
		// CONTROL MODE
    	if(r_cmode==ML_ON)
		{
			// decode word # and save as readen control word
	  		switch(_r_wcount)
	   		{
	     		case 0:
					cmode_rd_w00=_r_data;
					break;

	     		case 1:
					cmode_rd_w01=_r_data;
					break;

	     		case 2:
					cmode_rd_w02=_r_data;
					break;

	     		case 3:
					cmode_rd_w03=_r_data;
					break;

	     		case 4:
					cmode_rd_w10=_r_data;
					break;

	     		case 5:
					cmode_rd_w11=_r_data;
					break;

	     		case 6:
					cmode_rd_w12=_r_data;
					break;

	     		case 7:
					cmode_rd_w13=_r_data;
					break;

	     		case 8:
					cmode_rd_w20=_r_data;
					break;

	     		case 9:
					cmode_rd_w21=_r_data;
					break;

	     		case 10:
					cmode_rd_w22=_r_data;
					break;

	     		case 11:
					cmode_rd_w23=_r_data;
					break;

	     		case 12:
					cmode_rd_w30=_r_data;
					break;

	     		case 13:
					cmode_rd_w31=_r_data;
					break;

	     		case 14:
					cmode_rd_w32=_r_data;
					break;

	     		case 15:
					cmode_rd_w33=_r_data;
					break;
	   		}

			// check for receiving is complete,
			// If completed, then clear R_CMODE flag for background control
	   		if(num_frame==_r_wcount)	r_cmode=ML_OFF;
	 	}
	}
    else
    {
		// DATA mode
	  	switch(_r_wcount)     		// decode word # and save the word
	   	{
	     	case 0:					// save as ADC channel #0
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   		*adc_save_ptr = _r_data;
	      		break;

	     	case 1:					// save as ADC channel #1
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   		adc_save_ptr[adc_data_count_max] = _r_data;
	       		break;

	     	case 2:					// save as dmode_rd_w02
	       		dmode_rd_w02 = _r_data;
	       		break;

	     	case 3:					// save as DMODE_RD_W03
	       		dmode_rd_w03 = _r_data;
	       		break;

	     	case 4:					// save as ADC channel #2
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   		adc_save_ptr[2*adc_data_count_max] = _r_data;
	      		break;

	     	case 5:					// save as ADC channel #3
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   		adc_save_ptr[3*adc_data_count_max] = _r_data;
	       		break;

	     	case 6:					// save as dmode_rd_w12
	       		dmode_rd_w12 = _r_data;
	       		break;

	     	case 7:					// save as DMODE_RD_W13
	       		dmode_rd_w13 = _r_data;
	       		break;

	     	case 8:					// save as ADC channel #4
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   		adc_save_ptr[4*adc_data_count_max] = _r_data;
	      		break;

	     	case 9:					// save as ADC channel #5
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   		adc_save_ptr[5*adc_data_count_max] = _r_data;
	       		break;

	     	case 10:					// save as dmode_rd_w22
	       		dmode_rd_w22 = _r_data;
	       		break;

	     	case 11:					// save as DMODE_RD_W23
	       		dmode_rd_w23 = _r_data;
	       		break;

	     	case 12:					// save as ADC channel #6
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_0)!=0)
			   		adc_save_ptr[6*adc_data_count_max] = _r_data;
	      		break;

	     	case 13:					// save as ADC channel #7
	      		if ((adc_scan_flag==ML_ON) && (scan_channels & CHAN_1)!=0)
			   		adc_save_ptr[7*adc_data_count_max] = _r_data;
	       		break;

	     	case 14:					// save as dmode_rd_w32
	       		dmode_rd_w32 = _r_data;
	       		break;

	     	case 15:					// save as DMODE_RD_W33
	       		dmode_rd_w33 = _r_data;
	       		break;
	   	}

		// update pointers and counters
	   	if((num_frame==_r_wcount) && (adc_scan_flag==ML_ON))
        {
			adc_save_ptr++;

            if(++adc_data_count >=adc_data_count_max)
		    {
			  	if(adc_cyclic_scan_flag==ML_ON)
		      	{
					adc_data_count=0;
                	adc_save_ptr=adc_data_start_ptr;
		      	}
		      	else
		        	adc_scan_flag=ML_OFF;
		    }
		}
    }

    _r_wcount++;					// increment receiver word counter
	_r_wcount %=(num_frame+1);

   _TIME_TRACKING_SET_0;        	// clear PERFORMACE TRACKING FLAG
}
//-----------------------------------------------------------------------


//--------------------- programmable time delay functions ---------------
// S/w delay functions are used to provide required delays at the run-time.
// Since delays are generated via s/w, then two different set of delay functions
// are available:
//		- for C64xx DSP (running at 600/720MHz)
//		- and C620x/C6701/C6713B DSP running at 167..300MHz

void __delay(n)
{
	int i;

	for(i=n;i>=0;i--);
}

// this delay is used in order to ensure that McBSP port
// has been synchronized properly after configuration complete
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_delay_mcbsp_setup()		__delay(1000)
#else
	#define	_delay_mcbsp_setup()		__delay(300)
#endif

// this delay is used for short synchronization of McBSP port (typically two periods of CLKX/CLKR)
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_delay_mcbsp_setup_short()	__delay(600)
#else
	#define	_delay_mcbsp_setup_short()	__delay(200)
#endif

// this delay is used in the end of FSX cycle during A/C CONTROL mode
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_delay_100us()				__delay(10000)
#else
	#define	_delay_100us()				__delay(3000)
#endif

// this delay is used to ensure that AD1849 has passed over the autocalibration procedure
// after codec data mode has been set
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_wait_autocal_complete()	__delay(300000)
#else
	#define	_wait_autocal_complete()	__delay(100000)
#endif

// this delay is used after set/release of DCM reste signal
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_wait_for_dcm_reset()		__delay(300000)
#else
	#define	_wait_for_dcm_reset()		__delay(100000)
#endif

//-----------------------------------------------------------------------



//------------------ configuration of McBSP serial port -----------------

#define	MCBSP_SPCR_GO_SRGR_MASK		0x00400000
#define	MCBSP_SPCR_GO_FRAME_MASK	0x00800000

#define	MCBSP_SPCR_GO_RCV_MASK		0x00000001
#define	MCBSP_SPCR_GO_XMIT_MASK		0x00010000

#define	MCBSP_SPCR_RRDY_MASK		0x00000002


	// macros to enable/disable FSG during CMODE transmission
#define MCBSP_FSX_ENABLE			{_SIO_SPCR |= MCBSP_SPCR_GO_FRAME_MASK;}
#define	MCBSP_FSX_DISABLE			{_SIO_SPCR &= ~MCBSP_SPCR_GO_FRAME_MASK;}


//
// McBSP configuration for A/C CONTROL MODE
//	FSX=out XCLK=int FSR=ext RCLK=ext
//	XLEN=RLEN=16  XINT=RINT=enable
//	number of R/X 16-bit words/frame: 4x/8x/16x (defined by the number of enabled AD/DA channels)
//	serial_clock_period = CPU_CLK_period*48
//	frame_sync_period   = serial_clock_period*1024
//
//		note!!! FSYNC period is not important, since transmission is stopped after
//		        full frame is transmitted. FSYNC period must be larger than max
//				transmission time.

#define	MCBSP_SPCR_CMODE_GO_MASK	0x00010001

#define	MCBSP_SPCR_CMODE			0x00000000
#define	MCBSP_PCR_CMODE				0x00000a00

#define	MCBSP_XCR_CMODE_AC2CH		0x00050340
#define	MCBSP_XCR_CMODE_AC4CH		0x00050740
#define	MCBSP_XCR_CMODE_AC8CH		0x00050f40
#define	MCBSP_RCR_CMODE_AC2CH		0x00050340
#define	MCBSP_RCR_CMODE_AC4CH		0x00050740
#define	MCBSP_RCR_CMODE_AC8CH		0x00050f40

#define	MCBSP_SRGR_CMODE			0x34000064	// CLKGDV=100 (1.5MHz..3MHz), FSPER=1024


void init_sio_cmode()
// Configures DSP on-chip McBSP port for A/C CONTROL mode.
// In A/C CONTROL mode, DSP generates CLKX(CLKR) and FSX(FSR) via McBSP SRGR and
// FSG logic. CLKX/CLKR frequency is set approximately 5MHz for 200MHz C6701 DSP.
// FSX is generated by McBSP FSG as enabled by s/w. To do this, FSG is normally
// disabled in order to keep FSX=0, however when FSX must be generated, FRG is
// enabled via SPCR. FSG period must be long enough in order to exclude generation
// of FSX while FRG is enabled. FSR is on-board connected to FSX, and CLKR is
// connected to CLKX.
{
	_SIO_SPCR	= MCBSP_SPCR_CMODE  ;

	// provide delay to allow McBSP port to stop operation
	_delay_mcbsp_setup();

	_SIO_PCR	= MCBSP_PCR_CMODE  ;

	// set frame length in accordance with number of channels
	if(DAQ_CH2_MODE)
	{
	   	_SIO_XCR = MCBSP_XCR_CMODE_AC2CH  ;
		_SIO_RCR = MCBSP_RCR_CMODE_AC2CH  ;
	}

	if(DAQ_CH4_MODE)
	{
	   	_SIO_XCR = MCBSP_XCR_CMODE_AC4CH  ;
		_SIO_RCR = MCBSP_RCR_CMODE_AC4CH  ;
	}

	if(DAQ_CH8_MODE)
	{
    	_SIO_XCR = MCBSP_XCR_CMODE_AC8CH  ;
		_SIO_RCR = MCBSP_RCR_CMODE_AC8CH  ;
	}


	_SIO_SRGR = MCBSP_SRGR_CMODE;
	// provide delay to allow SRGR to synchronize
	_delay_mcbsp_setup();

	// enable SRGR operation
	_SIO_SPCR |= MCBSP_SPCR_GO_SRGR_MASK ;
	// provide delay to allow McBSP port to synchronize
	_delay_mcbsp_setup();

		// enable McBSP-0 operation
	_SIO_SPCR |= MCBSP_SPCR_CMODE_GO_MASK ;
	// provide delay to allow McBSP port to synchronize
	_delay_mcbsp_setup();
}



// McBSP port configuration for DATA MODE
//	RJUST=00, FSX=ext XCLK=ext FSR=ext RCLK=ext
//	XLEN=RLEN=16  XINT=RINT=enable
//	number of R/X 16-bit words/frame: 4x/8x/16x (defined by the number of enabled AD/DA channels)

#define	MCBSP_SPCR_DMODE_GO_MASK	0x00010001

#define	MCBSP_SPCR_DMODE			0x00000000
#define	MCBSP_PCR_DMODE				0x00000000
#define	MCBSP_XCR_DMODE_AC2CH		0x00050340
#define	MCBSP_XCR_DMODE_AC4CH		0x00050740
#define	MCBSP_XCR_DMODE_AC8CH		0x00050f40
#define	MCBSP_RCR_DMODE_AC2CH		0x00050340
#define	MCBSP_RCR_DMODE_AC4CH		0x00050740
#define	MCBSP_RCR_DMODE_AC8CH		0x00050f40

#define	MCBSP_SRGR_DMODE			0x20000000


void init_sio_dmode()
// Configures DSP on-chip McBSP port for A/C DATA-mode.
// In A/C DATA mode, A/C generates CLKX/CLKR and FSX/FSR.
// On exit, R/X-portions of McBSP port shall be kept in the RESET state
// in order the calling function can enable R/X-portions upon the
// synchronization events.
{

	_SIO_SPCR  	= MCBSP_SPCR_DMODE  ;

	// provide delay to allow McBSP port to stop operation
	_delay_mcbsp_setup();

	_SIO_PCR	= MCBSP_PCR_DMODE  ;

	// set frame length in accordance with number of channels
	if(DAQ_CH2_MODE)
	{
	   	_SIO_XCR = MCBSP_XCR_DMODE_AC2CH  ;
		_SIO_RCR = MCBSP_RCR_DMODE_AC2CH  ;
	}

	if(DAQ_CH4_MODE)
	{
    	_SIO_XCR = MCBSP_XCR_DMODE_AC4CH  ;
		_SIO_RCR = MCBSP_RCR_DMODE_AC4CH  ;
	}

	if(DAQ_CH8_MODE)
	{
    	_SIO_XCR = MCBSP_XCR_DMODE_AC8CH  ;
		_SIO_RCR = MCBSP_RCR_DMODE_AC8CH  ;
	}

	_SIO_SRGR = MCBSP_SRGR_DMODE;

	// provide delay to allow McBSP port to synchronize
	_delay_mcbsp_setup();
}
//-----------------------------------------------------------------------



//----------------- A/C configuration functions -------------------------

void set_ac_config_data()
// fills-in configuration data for CS4215/AD1849 A/C chips
{
 	// set number of channels
	scan_channels	= MCD->chan & 0xff;

	// each channel (#0..7) has its own enable bit
	// configure run-time params in accordance with enabled channels (2x/4x/8x)
	if(scan_channels>0xf)
	{
		// set params for 8x AD/DA channels
	    frame_len	= 2;
	    num_frame	= 15;
	}
	else
	{
		if(scan_channels>0x3)
	    {
			// set params for 4x AD/DA channels
	      	frame_len	= 1;
	      	num_frame	= 7;
	    }
	    else
	     {
		 	// set params for 2x AD/DA channels
			frame_len	= 0;
	      	num_frame	= 3;
	    }
	}

	// fill-in A/C data structures
		// create CMODE words #0/#1 and DMODE word #1
	t_ac_cts_1_2.clb=0;     		// CLB bit =0
	t_ac_cts_1_2.olb=0;				// select 1Vrms scale
	t_ac_cts_1_2.mlb=0;				// enable MIC +20dB amplifier

    t_ac_cts_1_2.rsrv3=1;			// reserved bits
	t_ac_cts_1_2.rsrv2=0;
	t_ac_cts_1_2.rsrv1=0;

	t_ac_cts_1_2.df = 0;			// 16-bit data
	t_ac_cts_1_2.st	= 1;			// stereo mode
	t_ac_cts_1_2.hpf = 0;			// disable high-pass filter

	// set CLK selection
	t_ac_cts_1_2.dfr = (MCD->fq & 0x7);
	t_acm_cts_3_4.mck = (((MCD->fq)>>3) & 0x1) +1;

	t_acm_cts_3_4.xen = 0;
	t_acm_cts_3_4.xclk= 1;
	t_acm_cts_3_4.bsel= frame_len;
	t_acm_cts_3_4.its = 1;
	t_acm_cts_3_4.dad = 0;			// disable loopback and tests
	t_acm_cts_3_4.enl = 0;
	t_acm_cts_3_4.test= 0;

	t_acs_cts_3_4.mck = 3;

	t_acs_cts_3_4.xen = 0;
	t_acs_cts_3_4.xclk= 0;
	t_acs_cts_3_4.bsel= frame_len;
	t_acs_cts_3_4.its = 1;
	t_acs_cts_3_4.dad = 0;			// disable loopback and tests
	t_acs_cts_3_4.enl = 0;
	t_acs_cts_3_4.test= 0;

	t_ac_cts_5_6.rsrv1 =0;
	t_ac_cts_5_6.rsrv2 =0;
	t_ac_cts_5_6.pio   =0;			// PIO unused

	t_ac_cts_7_8.rsrv1 =0;
	t_ac_cts_7_8.rsrv2 =0;
	t_ac_cts_7_8.ver   =0;  		// read-back the version value

	t_ac_dts_5_6.lo    =MCD->lo;
	t_ac_dts_5_6.le    =MCD->le;
	t_ac_dts_5_6.he    =MCD->he;
	t_ac_dts_5_6.ro    =MCD->ro;
	t_ac_dts_5_6.se    =0;			// speaker disable
	t_ac_dts_5_6.adi   =0;

	t_ac_dts_7_8.lg    =MCD->lg;
	t_ac_dts_7_8.is    =MCD->is;
	t_ac_dts_7_8.pio   =0;
	t_ac_dts_7_8.ovr   =0;

	t_ac_dts_7_8.rg    =MCD->rg;
	t_ac_dts_7_8.ma    =MCD->ma;


	// now convert A/C data structures to 16-bit UNSIGNED data for further transmission over McBSP port
	cmode_w0	=  *(unsigned short*) &t_ac_cts_1_2;

	cmode_w1_m	=  *(unsigned short*) &t_acm_cts_3_4;
	cmode_w1_s	=  *(unsigned short*) &t_acs_cts_3_4;
	cmode_w2	=  *(unsigned short*) &t_ac_cts_5_6;
	cmode_w3	=  *(unsigned short*) &t_ac_cts_7_8;

	// DMODE_W02 and DMODE_W03 will be left unchanged for T/SDAS-AU/8L DCM and will be
	// modified in background for T/SDAS-ATEL2 in order to provide copy of DIN-0..3 to
	// DOUT-0..3 digital I/O bits.
	dmode_w02	=  *(unsigned short*) &t_ac_dts_5_6;
	dmode_w03	=  *(unsigned short*) &t_ac_dts_7_8;

	dmode_w12	=  dmode_w02;
	dmode_w13	=  dmode_w03;
	dmode_w22	=  dmode_w02;
	dmode_w23	=  dmode_w03;
	dmode_w32	=  dmode_w02;
	dmode_w33	=  dmode_w03;
}




void rx_cmode_ccl()
// initializes transmission of all command data words in CONTROL modes
{
    // set CMODE flags for R/X in order to synchronize with R/X interrupt handlers
	// R/X interrupt handlers will clear these flags as soon as they will transmit
	// and receive required number of words.
	// Note, that _X_COUNT and _R_COUNT shall not be effected by this background
	// task, since they are updated by R/X interrupt handlers. _X_COUNT parameter
	// is typically '+2' ahead of this s/w position. _R_COUNT, however, can be
	// cleared here for  safe synchronization.
	x_cmode=r_cmode=ML_ON;

	// provide small delay to ensure correct sychronization
	_delay_mcbsp_setup();

	// enable FSX generator for McBSP in order to generate one FSX strobe
    MCBSP_FSX_ENABLE;

	// wait for CMODE_W0/W1 frames transferred and received
	WAIT_CMODE_RX_OK;

	// Disable FSG operation in order to exclude secondary FSX generation.
	// Note, that programmed FSG period must be large enough in order not to miss
	// this operation.
    MCBSP_FSX_DISABLE;

	// wait 100us to ensure that all data has been transferred
	_delay_100us();
}




int configure_ac()
// this function is used to generate config info for AC, configures AC in CONTROL mode and then
// goes to the DATA mode with DAC-scan and ADC-scan disabled.
// Note, that the timing procurement in this function is critical and clear timing understanding is
// required at the user side in order to have A/C chip to be configured properly in CONTROL mode
// and to initialize A/C DATA mode. Detail comments below are provided for all time critical operations.
{
	int	ccl_count;

	// assemble AC configuration CONTROL/DATA bit slots
    set_ac_config_data();

	// set CLB=0 for AC
	CLEAR_CLB;

	// set D/C control line to CONTROL mode
	SET_AC_CMODE;

	// release SIOX-A RESET signal for DCM and wait some time to ensure that A/C chip has get out of the reset
   	_REMOVE_DCM_RESET;
	_wait_for_dcm_reset();

	// clear R/X interrupt requests
	_MCBSP_RINT_CLEAR_;
	_MCBSP_XINT_CLEAR_;

	// configure DSP SIO-0 port
	init_sio_cmode();

 	// initialize R/X interrupt handlers parameters
 	// Note, that these parameters are updated by R/X interrupt handlers only and
 	// shall not be used by background task.
	_x_wcount=0;
	_r_wcount=0;

	// clear and enable DSP interrupts
	_MCBSP_RINT_ENABLE_;
	_MCBSP_XINT_ENABLE_;

	C6X_GINT_ENABLE;

	// perform two initial C-MODE transmit/receive cycles
	rx_cmode_ccl();
	rx_cmode_ccl();


	// check for the returned CLB bit. If not set, then
	// repeat transmission max 100 times.
	// (This meets specs for CS4215 and AD1849 ACs).
    if(   ((cmode_rd_w00 & _cmode_CLB_mask)!=0)
	   || (DAQ_CH4_OR_CH8_MODE && ((cmode_rd_w10 & _cmode_CLB_mask)!=0))
	   || (DAQ_CH8_MODE        && ((cmode_rd_w20 & _cmode_CLB_mask)!=0))
	   || (DAQ_CH8_MODE        && ((cmode_rd_w30 & _cmode_CLB_mask)!=0))
	  )
	{
	 	for(ccl_count=0;ccl_count<100;ccl_count++)
		{
		 	rx_cmode_ccl();
		}

		// check for CLB again. If still set, then return the HARDWARE ERROR.
  	    if(   	((cmode_rd_w00 & _cmode_CLB_mask)!=0)
			|| 	(DAQ_CH4_OR_CH8_MODE && ((cmode_rd_w10 & _cmode_CLB_mask)!=0))
		   	|| 	(DAQ_CH8_MODE        && ((cmode_rd_w20 & _cmode_CLB_mask)!=0))
	   		|| 	(DAQ_CH8_MODE        && ((cmode_rd_w30 & _cmode_CLB_mask)!=0))
	      )
			return(MERR_HW_ERR);
	}

	// continuously write control info to AC and read-back until
	// readen info will match the written one.
	do
	{
   	   rx_cmode_ccl();
	}
	while(   ((cmode_rd_w00 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      || ((cmode_rd_w01 & _cmode_rd_w1_mask)!=(cmode_w1_m & _cmode_rd_w1_mask))
	      || ((cmode_rd_w02 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      || ((cmode_rd_w03 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))

		  || (DAQ_CH4_OR_CH8_MODE &&
	      	   (   ((cmode_rd_w10 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      		|| ((cmode_rd_w11 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w12 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w13 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))
			   )
			 )

		  || (DAQ_CH8_MODE &&
		       (   ((cmode_rd_w20 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
				|| ((cmode_rd_w21 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w22 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w23 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))

	      		|| ((cmode_rd_w30 & _cmode_rd_w0_mask)!=(cmode_w0 & _cmode_rd_w0_mask))
	      		|| ((cmode_rd_w31 & _cmode_rd_w1_mask)!=(cmode_w1_s & _cmode_rd_w1_mask))
	      		|| ((cmode_rd_w32 & _cmode_rd_w2_mask)!=(cmode_w2 & _cmode_rd_w2_mask))
	      		|| ((cmode_rd_w33 & _cmode_rd_w3_mask)!=(cmode_w3 & _cmode_rd_w3_mask))
			   )
			 )
       );

	// now transmit at least 2 frames with CLB=1
	// in order to lock AC and to prepare to go to the DATA mode
	SET_CLB;

    rx_cmode_ccl();
    rx_cmode_ccl();
    rx_cmode_ccl();
    rx_cmode_ccl();

	// This is the end of A/C CONTROL mode. Once we get here, then we can be sure
	// that all A/C control parameters have been set properly.

	// now disable interrupts and go to the A/C DATA mode
	C6X_GINT_DISABLE;

	// We now have to switch A/C chip to the DATA mode via SIOX TM/XIO line,
	// reconfigure McBSP serial port, re-configure R/X interrupt handlers,
	// and to properly synchronize transmitter (X) interrupt handler and the
	// McBSP port transmitter.
	//
	// Notes:
	// ------
	//		1. 	In A/C DATA mode, CLKX/CLKR and FSX/FSR are generated by
	// 			the A/C chip, so we must disable McBSP port SRGR/FSG logic before or
	// 			immediately after A/C DATA mode is set.
	//		2.	It is required that X-interrupt handler is correctly synchronized with
	//			FSX signal, which is generated by A/C chip in the DATA mode. This means
	//			that McBSP port must be reconfigured for A/C DATA mode and initial data
	//			shall be written to McBSP' DXR register by X-interrupt (in order to 
	//			fill-in the XSR	transmitter shift register) exactly somewhere between 
	//			FSX/FSR pulses in order first FSX occured after McBSP re-initialization 
	//			will shift out valid serial data. Therefore, FSX occurance event shall 
	//			be safely detected.

	// disable AD/DA data acqisition enable flags for X/R inerrupt handlers
	// until they will be enabled by the background task later
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	// initialize R/X interrupt handlers parameters
	_x_wcount=0;
    _r_wcount=0;


	// Apply reset to the McBSP port in order to disable CLKX (CLKR) generator
	// and FSX (FSR) generator.
	_SIO_RESET;
	_delay_mcbsp_setup_short();

	// set A/C D/C control to the DATA mode
	SET_AC_DMODE;
	// wait to ensure AUTOCAL is completed
	_wait_autocal_complete();


	// configure DSP SIO-0 port for A/C data mode
	// Note, that R/X-portions of McBSP port will be held in the RESET state on
	// return from this fuction.
	init_sio_dmode();

	// We must now detect the FSX pulse in order to correctly synchronize this
	// background task with the X-interrupt handler. Synchronization includes
	// re-reset and enable of R/X-portions of McBSP port, and enable of McBSP 
	// X/R-interrupts. It is important to allow first X-interrupt to occur prior
	// in order to fill-in XSR register (via DXR) with valid data prior next FSX will
	// appear.

    	// We'll detect FSX event via the R-portion of McBSP port. To do this, we must
		// enable receiver operation and poll RRDY bit of McBSP SPCR register until
		// first R-data will become available. This time event will corresponds to
		// the end of first 16-bit word inside 64/128/256-bit A/C data frame. We'll
		// have the time equal to transmission of three 16-bit datawords in order to
		// perform McBSP port synchronization.

			// enable McBSP R/X operation
	_SIO_SPCR |= MCBSP_SPCR_GO_RCV_MASK ;

			// wait until first 16-bit will be received
	while( !(_SIO_SPCR & MCBSP_SPCR_RRDY_MASK));

			// apply reset to both R/X-portions
	_SIO_SPCR &= ~MCBSP_SPCR_GO_RCV_MASK ;
			// wait some time to ensure that both X/R-portions of McBSP have been put to the RESET state
			// (Note, that the wait time must not exceed the time till next FSX for the highest sampling frequency.)
	_delay_mcbsp_setup_short();

			// clear R/X interrupt requests
	_MCBSP_RINT_CLEAR_;
	_MCBSP_XINT_CLEAR_;

			// wait again to ensure that R/X interrupts are cleared prior X/R-portions of McBSP port will
			// be released from the RESET state
	_delay_mcbsp_setup_short();

			// enable McBSP R/X operation by releasing R/X-potions from the RESET state
	_SIO_SPCR |= MCBSP_SPCR_DMODE_GO_MASK ;

			// finally, enable DSP interrupts in order to enable
			// R/X-interrupt handlers to run
	C6X_GINT_ENABLE;

	// Note, that two X-interrupts will occur right here after global interrupts
	// are enabled. Sufficient time must be provided for the first-only X-interrupt
	// to occur in order to fill-in XSR register with valid data prior next FSX
	// pulse will appear. Second X-interrupt will fill-in DXR register.

	return(MERR_OK);

}
//-----------------------------------------------------------------------



//---------------------------- DAQ functions ----------------------------

int	das_dac_scan()
// this procedure performs cyclic DAC output.
{
	int r;

  	// disable interrupts
	C6X_GINT_DISABLE;

	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	// configure A/C chip and go to the DATA mode
   	r=configure_ac();

	// terminate if error is set
	if(r!=MERR_OK)
	{
		C6X_GINT_DISABLE;
	    return(r);
	}

	// set X/R interrupt driver parameters
	dac_data_start_ptr		= DAC;
    dac_get_ptr				= DAC;
	dac_data_count			= 0;
    dac_data_count_max		= MCD->epoch_len;

	adc_data_start_ptr		= ADC;
    adc_save_ptr			= ADC;
	adc_data_count			= 0;
    adc_data_count_max		= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag			= ML_OFF;
	dac_scan_flag			= ML_ON;

	// loop forever
	while(1);
}




int	adc_to_dac()
// this procedure performs cyclic ADC->DAC data copy (pass-through channel)
{
	int r;

	// disable interrupts
	C6X_GINT_DISABLE;

	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	// configure A/C chip and go to the DATA mode
   	r=configure_ac();

	// terminate if error is set
	if(r!=MERR_OK)
	{
		C6X_GINT_DISABLE;
	    return(r);
	}

	// set X/R interrupt driver parameters
	dac_data_start_ptr		= ADC;
    dac_get_ptr				= ADC;
	dac_data_count			= 0;
    dac_data_count_max		= 0x10; 	// use 10-words cyclic buffer for ADC->DAC copy


	adc_data_start_ptr		= ADC;
    adc_save_ptr			= ADC;
	adc_data_count			= 0;
    adc_data_count_max		= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	dac_scan_flag			= ML_ON;
	adc_scan_flag			= ML_ON;

	// loop forever
	while(1)
	{
#if(_DCM_TSDAS_ATEL2_SELECTED)
		// 	For T/SDAS-ATEL2 DCM with DOUT-0..3 and DIN-0..3 digital I/O bits, perform
		//	copy of DIN-0..3 digital inputs to DOUT-0..3 digital outputs. This copy is
		//	performed in background using dmode_rd_w02/dmode_rd_w03 input data words
		//	and dmode_w02/dmode_w03 output data words.
		//	DOUT-0/1 and DIN-0/1 bits are allocated to bits D14/D15 of DMODE_W02 and DMODE_RD_W02
		//	16-bt datawords, whereas DOUT-2/3 and DIN-2/3 bits are allocated to bits D14/D15 of
		//	DMODE_W03 and DMODE_RD_W03 data words.

		// 	perform copy of DIN-0..3 bits to DOUT-0..3 bits
		//  Note, that register read/write must be perform only once in order to exclude data alternation.
	   	ww2 = (dmode_w02 & ~_atel2_dio01_mask) ;
   		ww3 = (dmode_w03 & ~_atel2_dio23_mask) ;

	   	rw2 = (dmode_rd_w02 &  _atel2_dio01_mask) ;
   		rw3 = (dmode_rd_w03 &  _atel2_dio23_mask) ;

	   	ww2 = (ww2 | rw2);
   		ww3 = (ww3 | rw3);

	   	dmode_w02 = ww2;
   		dmode_w03 = ww3;

#endif
	}
}
//-----------------------------------------------------------------------

